@@ -21,6 +21,7 @@ LIBS=(
 	libffi-7.dll\
 	libiconv-2.dll\
 	libpcre-1.dll\
+	libpcre2-8-0.dll\
 	libatkmm-1.6-1.dll\
 	libatk-1.0-0.dll\
 	libgtk-3-0.dll\
@@ -92,8 +93,10 @@ LIBS=(
 	libcrypto-1_1-x64.dll\
 	libpodofo.dll\
 	libidn-12.dll\
-	libzip.dll\
+	libarchive-13.dll\
 	libzstd.dll\
+	liblz4.dll\
+	libb2-1.dll\
 	libssp-0.dll\
 	libdeflate.dll\
 	libwebp-7.dll\
