@@ -33,5 +33,5 @@ fi
 echo "Using clang-format command: ${CLANG_BIN}"
 
 # Search using the binary found
-find ./src \( -iname *.h -o -iname *.cpp -o -iname *.hpp -o -iname *.cc -o -iname *.c \) ! -iname bitmap_font_*.c | xargs $CLANG_BIN -style=file -i
+find ./src \( -iname *.h -o -iname *.cpp -o -iname *.hpp -o -iname *.cc -o -iname *.c \) ! -iname bitmap_font_*.c ! -iname hershey_fonts.cpp | xargs $CLANG_BIN -style=file -i
 exit $?
