@@ -4,6 +4,7 @@
 #include "pool/part.hpp"
 #include "nlohmann/json.hpp"
 #include "pool/ipool.hpp"
+#include "util/pin_direction_accumulator.hpp"
 
 namespace horizon {
 
@@ -34,6 +35,72 @@ Connection::Connection(const json &j, Block *block)
         net.uuid = j.at("net").get<std::string>();
 }
 
+
+Pin::Direction Component::get_effective_direction(const Component::AltPinInfo &alt, const Pin &pin)
+{
+    if (alt.use_custom_name)
+        return alt.custom_direction;
+    PinDirectionAccumulator acc;
+    if (alt.use_primary_name)
+        acc.accumulate(pin.direction);
+    for (const auto &n : alt.pin_names) {
+        if (pin.names.count(n))
+            acc.accumulate(pin.names.at(n).direction);
+    }
+    return acc.get().value_or(pin.direction);
+}
+
+Pin::Direction Component::get_effective_direction(const UUIDPath<2> &path) const
+{
+    const auto &pin = entity->gates.at(path.at(0)).unit->pins.at(path.at(1));
+    if (alt_pins.count(path))
+        return get_effective_direction(alt_pins.at(path), pin);
+    else
+        return pin.direction;
+}
+
+
+void Component::AltPinInfo::update_for_index(int index, const Pin &pin)
+{
+    if (index == -2) {
+        use_custom_name = true;
+    }
+    else if (index == -1) {
+        use_primary_name = true;
+    }
+    else {
+        const auto uu = Pin::alternate_name_uuid_from_index(index);
+        if (pin.names.count(uu))
+            pin_names.insert(uu);
+    }
+}
+
+Component::AltPinInfo::AltPinInfo(const json &j, const Pin &pin)
+    : use_primary_name(j.at("use_primary_name").get<bool>()), use_custom_name(j.at("use_custom_name").get<bool>()),
+      custom_name(j.at("custom_name").get<std::string>()),
+      custom_direction(Pin::direction_lut.lookup(j.at("custom_direction").get<std::string>()))
+{
+    for (const auto &it : j.at("pin_names")) {
+        const UUID u{it.get<std::string>()};
+        if (pin.names.count(u))
+            pin_names.insert(u);
+    }
+}
+
+json Component::AltPinInfo::serialize() const
+{
+    json j;
+    j["use_primary_name"] = use_primary_name;
+    j["use_custom_name"] = use_custom_name;
+    j["custom_name"] = custom_name;
+    j["custom_direction"] = Pin::direction_lut.lookup_reverse(custom_direction);
+    j["pin_names"] = json::array();
+    for (const auto &it : pin_names)
+        j["pin_names"].push_back((std::string)it);
+    return j;
+}
+
+
 Component::Component(const UUID &uu, const json &j, IPool &pool, Block *block)
     : uuid(uu), entity(pool.get_entity(j.at("entity").get<std::string>())), refdes(j.at("refdes").get<std::string>()),
       value(j.at("value").get<std::string>()), nopopulate(j.value("nopopulate", false))
@@ -74,29 +141,42 @@ Component::Component(const UUID &uu, const json &j, IPool &pool, Block *block)
             }
         }
     }
-    {
-        const json &o = j["pin_names"];
-        for (const auto &[key, val] : o.items()) {
+
+    if (j.count("alt_pins")) { // new one
+        for (const auto &[key, val] : j.at("alt_pins").items()) {
             UUIDPath<2> u(key);
-            if (entity->gates.count(u.at(0)) > 0) {
+            if (entity->gates.count(u.at(0))) {
                 const auto &g = entity->gates.at(u.at(0));
-                if (g.unit->pins.count(u.at(1)) > 0) {
+                if (g.unit->pins.count(u.at(1))) {
                     const auto &p = g.unit->pins.at(u.at(1));
-                    std::set<int> s;
-                    if (val.is_number_integer()) {
-                        const auto index = val.get<int>();
-                        if ((int)p.names.size() >= index + 1) {
-                            s.insert(index);
-                        }
-                    }
-                    else if (val.is_array()) {
-                        s = val.get<decltype(s)>();
-                    }
-                    pin_names.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(s));
+                    alt_pins.emplace(std::piecewise_construct, std::forward_as_tuple(u), std::forward_as_tuple(val, p));
                 }
             }
         }
     }
+    else if (j.count("pin_names")) { // old one
+        for (const auto &[key, val] : j.at("pin_names").items()) {
+            UUIDPath<2> u(key);
+            if (entity->gates.count(u.at(0))) {
+                const auto &g = entity->gates.at(u.at(0));
+                if (g.unit->pins.count(u.at(1))) {
+                    const auto &p = g.unit->pins.at(u.at(1));
+                    auto &info = alt_pins[u];
+                    std::set<UUID> s;
+                    if (val.is_number_integer()) {
+                        const auto index = val.get<int>();
+                        info.update_for_index(index, p);
+                    }
+                    else if (val.is_array()) {
+                        for (const auto &x : val) {
+                            info.update_for_index(x.get<int>(), p);
+                        }
+                    }
+                }
+            }
+        }
+    }
+
     if (j.count("custom_pin_names")) {
         const json &o = j["custom_pin_names"];
         for (const auto &[key, val] : o.items()) {
@@ -104,7 +184,7 @@ Component::Component(const UUID &uu, const json &j, IPool &pool, Block *block)
             if (entity->gates.count(u.at(0)) > 0) {
                 const auto &g = entity->gates.at(u.at(0));
                 if (g.unit->pins.count(u.at(1)) > 0) {
-                    custom_pin_names.emplace(u, val.get<std::string>());
+                    alt_pins[u].custom_name = val.get<std::string>();
                 }
             }
         }
@@ -120,7 +200,6 @@ UUID Component::get_uuid() const
 {
     return uuid;
 }
-
 std::string Component::replace_text(const std::string &t, bool *replaced) const
 {
     if (replaced)
@@ -177,15 +256,10 @@ json Component::serialize() const
     for (const auto &it : connections) {
         j["connections"][(std::string)it.first] = it.second.serialize();
     }
-    j["pin_names"] = json::object();
-    for (const auto &it : pin_names) {
-        j["pin_names"][(std::string)it.first] = it.second;
-    }
-    j["custom_pin_names"] = json::object();
-    for (const auto &it : custom_pin_names) {
-        if (it.second.size()) {
-            j["custom_pin_names"][(std::string)it.first] = it.second;
-        }
+    j["pin_names"] = json::object(); // for compatibility
+    j["alt_pins"] = json::object();  // for compatibility
+    for (const auto &[k, v] : alt_pins) {
+        j["alt_pins"][(std::string)k] = v.serialize();
     }
     if (href.size())
         j["href"] = uuid_vec_to_string(href);
