@@ -10,6 +10,7 @@
 #include "pool/ipool.hpp"
 #include "common/junction_util.hpp"
 #include "util/bbox_accumulator.hpp"
+#include "util/polygon_arc_removal_proxy.hpp"
 
 namespace horizon {
 
@@ -33,7 +34,12 @@ BoardColors::BoardColors() : solder_mask({0, .5, 0}), silkscreen({1, 1, 1}), sub
 {
 }
 
-static const unsigned int app_version = 7;
+const LutEnumStr<Board::OutputFormat> Board::output_format_lut = {
+        {"gerber", Board::OutputFormat::GERBER},
+        {"odb", Board::OutputFormat::ODB},
+};
+
+static const unsigned int app_version = 14;
 
 unsigned int Board::get_app_version()
 {
@@ -215,6 +221,20 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
                          Logger::Domain::BOARD);
         }
     }
+    if (j.count("net_ties")) {
+        const json &o = j["net_ties"];
+        for (auto it = o.cbegin(); it != o.cend(); ++it) {
+            UUID net_tie_uuid(it.value().at("net_tie").get<std::string>());
+            auto u = UUID(it.key());
+            if (block->net_ties.count(net_tie_uuid)) {
+                load_and_log(net_ties, ObjectType::BOARD_NET_TIE, std::forward_as_tuple(u, it.value(), this),
+                             Logger::Domain::BOARD);
+            }
+            else {
+                Logger::log_warning("not loading net tie " + (std::string)u + " since net tie in netlist is gone");
+            }
+        }
+    }
     if (j.count("rules")) {
         try {
             rules.load_from_json(j.at("rules"));
@@ -226,12 +246,23 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
     }
     if (j.count("fab_output_settings")) {
         try {
-            fab_output_settings = FabOutputSettings(j.at("fab_output_settings"));
+            gerber_output_settings = GerberOutputSettings(j.at("fab_output_settings"));
         }
         catch (const std::exception &e) {
             Logger::log_warning("couldn't load fab output settings", Logger::Domain::BOARD, e.what());
         }
     }
+    if (j.count("odb_output_settings")) {
+        try {
+            odb_output_settings = ODBOutputSettings(j.at("odb_output_settings"));
+        }
+        catch (const std::exception &e) {
+            Logger::log_warning("couldn't load ODB++ output settings", Logger::Domain::BOARD, e.what());
+        }
+    }
+    if (j.count("output_format")) {
+        output_format = output_format_lut.lookup(j.at("output_format"));
+    }
     if (j.count("colors")) {
         try {
             const auto &o = j.at("colors");
@@ -277,7 +308,8 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
             Logger::log_warning("couldn't load grid settings", Logger::Domain::BOARD, e.what());
         }
     }
-    fab_output_settings.update_for_board(*this);
+    gerber_output_settings.update_for_board(*this);
+    odb_output_settings.update_for_board(*this);
     update_pdf_export_settings(pdf_export_settings);
     update_refs(); // fill in smashed texts
 }
@@ -327,7 +359,8 @@ Board::Board(const Board &brd, CopyMode copy_mode)
       junctions(brd.junctions), tracks(brd.tracks), texts(brd.texts), lines(brd.lines), arcs(brd.arcs),
       planes(brd.planes), keepouts(brd.keepouts), dimensions(brd.dimensions), connection_lines(brd.connection_lines),
       included_boards(brd.included_boards), board_panels(brd.board_panels), pictures(brd.pictures), decals(brd.decals),
-      warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings),
+      net_ties(brd.net_ties), warnings(brd.warnings), output_format(brd.output_format), rules(brd.rules),
+      gerber_output_settings(brd.gerber_output_settings), odb_output_settings(brd.odb_output_settings),
       grid_settings(brd.grid_settings), airwires(brd.airwires), stackup(brd.stackup), colors(brd.colors),
       pdf_export_settings(brd.pdf_export_settings), step_export_settings(brd.step_export_settings),
       pnp_export_settings(brd.pnp_export_settings), version(brd.version), n_inner_layers(brd.n_inner_layers)
@@ -412,6 +445,10 @@ void Board::update_refs()
     for (auto &it : board_panels) {
         it.second.included_board.update(included_boards);
     }
+    for (auto &it : net_ties) {
+        it.second.net_tie.update(block->net_ties);
+        it.second.update_refs(*this);
+    }
 }
 
 void Board::update_junction_connections()
@@ -421,6 +458,7 @@ void Board::update_junction_connections()
         it.connected_connection_lines.clear();
         it.connected_tracks.clear();
         it.connected_vias.clear();
+        it.connected_net_ties.clear();
         it.has_via = false;
         it.needs_via = false;
     }
@@ -450,6 +488,12 @@ void Board::update_junction_connections()
         it.junction->layer = LayerRange(BoardLayers::TOP_COPPER, BoardLayers::BOTTOM_COPPER);
         it.junction->connected_vias.push_back(uu);
     }
+    for (auto &[uu, it] : net_ties) {
+        for (auto &it_ft : {&it.from, &it.to}) {
+            (*it_ft)->connected_net_ties.push_back(uu);
+            (*it_ft)->layer.merge(it.layer);
+        }
+    }
 }
 
 unsigned int Board::get_n_inner_layers() const
@@ -490,7 +534,8 @@ void Board::set_n_inner_layers(unsigned int n)
         stackup.emplace(-j, -j);
     }
     map_erase_if(stackup, [this](const auto &x) { return layers.count(x.first) == 0; });
-    fab_output_settings.update_for_board(*this);
+    gerber_output_settings.update_for_board(*this);
+    odb_output_settings.update_for_board(*this);
     update_pdf_export_settings(pdf_export_settings);
 }
 
@@ -631,7 +676,7 @@ void Board::vacuum_junctions()
     map_erase_if(junctions, [](auto &x) {
         const BoardJunction &ju = x.second;
         return (ju.connected_lines.size() == 0) && (ju.connected_arcs.size() == 0) && (ju.connected_tracks.size() == 0)
-               && (ju.connected_vias.size() == 0);
+               && (ju.connected_vias.size() == 0) && (ju.connected_net_ties.size() == 0);
     });
 }
 
@@ -1096,7 +1141,9 @@ json Board::serialize() const
     j["name"] = name;
     j["n_inner_layers"] = n_inner_layers;
     j["rules"] = rules.serialize();
-    j["fab_output_settings"] = fab_output_settings.serialize();
+    j["fab_output_settings"] = gerber_output_settings.serialize();
+    j["odb_output_settings"] = odb_output_settings.serialize();
+    j["output_format"] = output_format_lut.lookup_reverse(output_format);
     {
         json o;
         o["solder_mask"] = color_to_json(colors.solder_mask);
@@ -1188,6 +1235,12 @@ json Board::serialize() const
             j["decals"][(std::string)it.first] = it.second.serialize();
         }
     }
+    if (net_ties.size()) {
+        j["net_ties"] = json::object();
+        for (const auto &it : net_ties) {
+            j["net_ties"][(std::string)it.first] = it.second.serialize();
+        }
+    }
     return j;
 }
 
@@ -1224,4 +1277,198 @@ ItemSet Board::get_pool_items_used() const
     return items_needed;
 }
 
+static Polygon transform_polygon(const Polygon &poly, const Placement &tr)
+{
+    Polygon out{UUID()};
+
+    out.vertices.reserve(poly.vertices.size());
+    for (const auto &v : poly.vertices) {
+        out.vertices.emplace_back(tr.transform(v.position));
+        auto &vo = out.vertices.back();
+        if (v.type == Polygon::Vertex::Type::ARC) {
+            vo.type = v.type;
+            vo.arc_center = tr.transform(v.arc_center);
+            vo.arc_reverse = v.arc_reverse != tr.mirror;
+        }
+    }
+    return out;
+}
+
+static ClipperLib::Path polygon_to_path(const Polygon &ipoly)
+{
+    PolygonArcRemovalProxy prx(ipoly);
+    const auto &vs = prx.get().vertices;
+    ClipperLib::Path out;
+    out.reserve(vs.size());
+    for (const auto &v : vs) {
+        out.emplace_back(v.position.x, v.position.y);
+    }
+
+    return out;
+}
+
+struct PolyInfo {
+    PolyInfo(const Polygon &p) : polygon(p), path(polygon_to_path(polygon)){};
+    Polygon polygon;
+    const ClipperLib::Path path;
+    bool encloses_all_others = true;
+
+    bool encloses(const PolyInfo &other) const
+    {
+        ClipperLib::Clipper clipper;
+        clipper.AddPath(other.path, ClipperLib::ptSubject, true);
+        clipper.AddPath(path, ClipperLib::ptClip, true);
+
+        ClipperLib::Paths result;
+        clipper.Execute(ClipperLib::ctDifference, result, ClipperLib::pftNonZero);
+        return result.size() == 0;
+    }
+
+    ClipperLib::Paths intersects(const PolyInfo &other) const
+    {
+        ClipperLib::Clipper clipper;
+        clipper.AddPath(other.path, ClipperLib::ptSubject, true);
+        clipper.AddPath(path, ClipperLib::ptClip, true);
+
+        ClipperLib::Paths result;
+        clipper.Execute(ClipperLib::ctIntersection, result, ClipperLib::pftNonZero);
+        return result;
+    }
+};
+
+Board::Outline Board::get_outline(bool with_errors) const
+{
+    std::vector<PolyInfo> polys;
+    for (const auto &[uu, it] : polygons) {
+        if (it.layer == BoardLayers::L_OUTLINE)
+            polys.emplace_back(it);
+    }
+    for (const auto &[uu_pkg, pkg] : packages) {
+        if (!pkg.omit_outline) {
+            for (const auto &[uu, it] : pkg.package.polygons) {
+                if (it.layer == BoardLayers::L_OUTLINE) {
+                    Placement tr = pkg.placement;
+                    if (pkg.flip)
+                        tr.invert_angle();
+                    polys.emplace_back(transform_polygon(it, tr));
+                }
+            }
+        }
+    }
+
+    if (polys.size() == 0) {
+        Outline out{UUID()};
+        if (with_errors) {
+            out.errors.errors.emplace_back(RulesCheckErrorLevel::FAIL, "Found no outline polygons");
+            out.errors.update();
+        }
+        return out;
+    }
+
+
+    if (polys.size() == 1) {
+        const auto &outline_poly = polys.front();
+        Board::Outline outline{outline_poly.polygon, {}};
+        // On Y-axis positive upward displays, Orientation will return true if the polygon's orientation is
+        // counter-clockwise.
+
+        if (ClipperLib::Orientation(outline_poly.path))
+            outline.outline.reverse();
+
+        return outline;
+    }
+
+    // one of polys must be enclosing all others
+    for (size_t i = 0; i < polys.size(); i++) {
+        for (size_t j = 0; j < polys.size(); j++) {
+            if (i != j) {
+                if (!polys.at(i).encloses(polys.at(j))) {
+                    polys.at(i).encloses_all_others = false;
+                }
+            }
+        }
+    }
+
+    size_t outline_idx = SIZE_MAX;
+    for (size_t i = 0; i < polys.size(); i++) {
+        if (polys.at(i).encloses_all_others) {
+            if (outline_idx == SIZE_MAX) {
+                outline_idx = i;
+            }
+            else {
+                Outline out{UUID()};
+                if (with_errors) {
+                    out.errors.errors.emplace_back(RulesCheckErrorLevel::FAIL,
+                                                   "More than one outline polygon encloses all others");
+                    out.errors.update();
+                }
+                return out;
+            }
+        }
+    }
+
+    if (outline_idx == SIZE_MAX) {
+        Outline out{UUID()};
+        if (with_errors) {
+            out.errors.errors.emplace_back(RulesCheckErrorLevel::FAIL, "No outline polygon encloses all others");
+            out.errors.update();
+        }
+        return out;
+    }
+
+    const auto &outline_poly = polys.at(outline_idx);
+    Board::Outline outline{outline_poly.polygon, {}};
+    // On Y-axis positive upward displays, Orientation will return true if the polygon's orientation is
+    // counter-clockwise.
+
+    if (ClipperLib::Orientation(outline_poly.path))
+        outline.outline.reverse();
+
+    // make sure that holes don't intersect
+    for (size_t i = 0; i < polys.size(); i++) {
+        for (size_t j = i + 1; j < polys.size(); j++) {
+            if (i != outline_idx && j != outline_idx) {
+                if (auto isect = polys.at(i).intersects(polys.at(j)); isect.size()) {
+                    Outline out{UUID()};
+                    if (with_errors) {
+                        for (const auto &path : isect) {
+                            out.errors.errors.emplace_back(RulesCheckErrorLevel::FAIL, "Hole intersects other hole");
+                            auto &x = out.errors.errors.back();
+                            x.has_location = true;
+                            x.location.x = path.front().X;
+                            x.location.y = path.front().Y;
+                            x.error_polygons = {path};
+                        }
+                        out.errors.update();
+                    }
+                    return out;
+                }
+            }
+        }
+    }
+
+    outline.holes.reserve(polys.size() - 1);
+    for (size_t i = 0; i < polys.size(); i++) {
+        if (i != outline_idx) {
+            const auto &p = polys.at(i);
+            outline.holes.emplace_back(p.polygon);
+            if (!ClipperLib::Orientation(p.path))
+                outline.holes.back().reverse();
+        }
+    }
+
+    return outline;
+}
+
+Board::Outline Board::get_outline() const
+{
+    return get_outline(false);
+}
+
+
+Board::Outline Board::get_outline_and_errors() const
+{
+    return get_outline(true);
+}
+
 } // namespace horizon
