@@ -11,7 +11,8 @@
 #include "common/polygon.hpp"
 #include "common/keepout.hpp"
 #include "common/pdf_export_settings.hpp"
-#include "fab_output_settings.hpp"
+#include "gerber_output_settings.hpp"
+#include "odb_output_settings.hpp"
 #include "nlohmann/json_fwd.hpp"
 #include "plane.hpp"
 #include "track.hpp"
@@ -28,6 +29,7 @@
 #include "board_decal.hpp"
 #include "util/file_version.hpp"
 #include "common/grid_settings.hpp"
+#include "board_net_tie.hpp"
 
 namespace horizon {
 using json = nlohmann::json;
@@ -40,6 +42,8 @@ public:
     Color substrate;
 };
 
+using plane_update_status_cb_t = std::function<void(const Plane &plane, const std::string &)>;
+
 class Board : public ObjectProvider, public LayerProvider {
 private:
     // unsigned int update_nets();
@@ -78,9 +82,12 @@ public:
     const std::map<int, Layer> &get_layers() const override;
     void set_n_inner_layers(unsigned int n);
     unsigned int get_n_inner_layers() const;
-    void update_plane(Plane *plane, const class CanvasPatch *ca = nullptr,
-                      const class CanvasPads *ca_pads = nullptr); // when ca is given, patches will be read from it
-    void update_planes();
+    void update_plane(
+            Plane *plane, const class CanvasPatch *ca = nullptr, const class CanvasPads *ca_pads = nullptr,
+            plane_update_status_cb_t status_cb = nullptr,
+            const std::atomic_bool &cancel = std::atomic_bool(false)); // when ca is given, patches will be read from it
+    void update_planes(plane_update_status_cb_t status_cb = nullptr,
+                       const std::atomic_bool &cancel = std::atomic_bool(false));
     std::vector<KeepoutContour> get_keepout_contours() const;
     std::pair<Coordi, Coordi> get_bbox() const;
     void update_pdf_export_settings(PDFExportSettings &settings);
@@ -107,11 +114,17 @@ public:
     std::map<UUID, BoardPanel> board_panels;
     std::map<UUID, Picture> pictures;
     std::map<UUID, BoardDecal> decals;
+    std::map<UUID, BoardNetTie> net_ties;
 
     std::vector<Warning> warnings;
 
+    enum class OutputFormat { GERBER, ODB };
+    OutputFormat output_format = OutputFormat::GERBER;
+    static const LutEnumStr<OutputFormat> output_format_lut;
+
     BoardRules rules;
-    FabOutputSettings fab_output_settings;
+    GerberOutputSettings gerber_output_settings;
+    ODBOutputSettings odb_output_settings;
     GridSettings grid_settings;
 
     std::map<UUID, std::list<Airwire>> airwires;
@@ -157,6 +170,16 @@ public:
     int get_package_layer(bool flip, int layer) const;
     ParameterSet get_parameters() const;
 
+    struct Outline {
+        Polygon outline;            // clockwise
+        std::vector<Polygon> holes; // counter-clockwise
+
+        RulesCheckResult errors;
+    };
+
+    Outline get_outline() const;
+    Outline get_outline_and_errors() const;
+
 private:
     unsigned int n_inner_layers = 0;
     ClipperLib::Paths get_thermals(class Plane *plane, const class CanvasPads *ca) const;
@@ -165,6 +188,7 @@ private:
 
     Board(const Board &brd, CopyMode copy_mode);
     void expand_packages();
+    Outline get_outline(bool with_errors) const;
 };
 
 inline Board::ExpandFlags operator|(Board::ExpandFlags a, Board::ExpandFlags b)
