@@ -6,9 +6,6 @@
 #include "nlohmann/json_fwd.hpp"
 #include "util/uuid.hpp"
 #include "util/uuid_ptr.hpp"
-#include <fstream>
-#include <map>
-#include <vector>
 
 namespace horizon {
 using json = nlohmann::json;
@@ -42,6 +39,7 @@ public:
         uuid_ptr<BoardJunction> junc = nullptr;
         uuid_ptr<BoardPackage> package = nullptr;
         uuid_ptr<Pad> pad = nullptr;
+        Coordi offset;
         bool operator<(const Track::Connection &other) const;
         bool operator==(const Track::Connection &other) const;
 
