@@ -18,13 +18,14 @@
 #include "picture.hpp"
 #include "pool/unit.hpp"
 #include "util/vector_pair.hpp"
-#include "util/text_data.hpp"
+#include "text_renderer.hpp"
 
 namespace horizon {
 class Canvas {
     friend Selectables;
     friend class SelectionFilter;
     friend class CanvasAnnotation;
+    friend CanvasTextRenderer;
 
 public:
     Canvas();
@@ -32,7 +33,8 @@ public:
     {
     }
     virtual void clear();
-    void update(const class Symbol &sym, const Placement &transform = Placement(), bool edit = true);
+    enum class SymbolMode { SHEET, EDIT, EDIT_PREVIEW };
+    void update(const class Symbol &sym, const Placement &transform = Placement(), SymbolMode mode = SymbolMode::EDIT);
     void update(const class Sheet &sheet);
     void update(const class Padstack &padstack, bool edit = true);
     void update(const class Package &pkg, bool edit = true);
@@ -102,11 +104,12 @@ protected:
     }
     void object_ref_pop();
 
-    void render(const class Symbol &sym, bool on_sheet = false, bool smashed = false, ColorP co = ColorP::FROM_LAYER);
+    void render(const class Symbol &sym, SymbolMode mode = SymbolMode::SHEET, bool smashed = false,
+                ColorP co = ColorP::FROM_LAYER);
     void render(const class Junction &junc, bool interactive = true, ObjectType mode = ObjectType::INVALID);
     void render(const class SchematicJunction &junc);
     void render(const class Line &line, bool interactive = true, ColorP co = ColorP::FROM_LAYER);
-    void render(const class SymbolPin &pin, bool interactive = true, ColorP co = ColorP::FROM_LAYER);
+    void render(const class SymbolPin &pin, SymbolMode mode = SymbolMode::EDIT, ColorP co = ColorP::FROM_LAYER);
     void render(const class Arc &arc, bool interactive = true, ColorP co = ColorP::FROM_LAYER);
     void render(const class Sheet &sheet);
     void render(const class SchematicSymbol &sym);
@@ -142,6 +145,8 @@ protected:
     void render(const class BlockSymbol &sym, bool on_sheet = false);
     void render(const class BlockSymbolPort &port, bool interactive = true);
     void render(const class SchematicBlockSymbol &sym);
+    void render(const class SchematicNetTie &tie);
+    void render(const class BoardNetTie &tie, bool interactive = true);
 
     bool needs_push = true;
     virtual void request_push() = 0;
@@ -160,22 +165,14 @@ protected:
     void draw_arc(const Coord<float> &center, float radius, float a0, float a1, ColorP color = ColorP::FROM_LAYER,
                   int layer = 10000);
     void draw_circle(const Coord<float> &center, float radius, ColorP color = ColorP::FROM_LAYER, int layer = 10000);
-    std::pair<Coordf, Coordf> draw_arc2(const Coord<float> &center, float radius0, float a0, float a1, ColorP color,
-                                        int layer, uint64_t width);
+    void draw_arc2(const Coord<float> &center, float radius0, float a0, float a1, ColorP color, int layer,
+                   uint64_t width);
     void draw_arc0(const Coord<float> &center, float radius0, float a0, float a1, ColorP color, int layer,
                    uint64_t width);
 
-    struct TextOptions {
-        uint64_t width = 0;
-        bool flip = false;
-        bool mirror = false;
-        bool draw = true;
-        TextData::Font font = TextData::Font::SIMPLEX;
-        bool center = false;
-        bool allow_upside_down = false;
-    };
+
     std::pair<Coordf, Coordf> draw_text(const Coordf &p, float size, const std::string &rtext, int angle,
-                                        TextOrigin origin, ColorP color, int layer, const TextOptions &opts);
+                                        TextOrigin origin, ColorP color, int layer, const TextRenderer::Options &opts);
 
     virtual void draw_bitmap_text(const Coordf &p, float scale, const std::string &rtext, int angle, ColorP color,
                                   int layer)
@@ -217,9 +214,17 @@ protected:
     {
     }
     virtual void img_line(const Coordi &p0, const Coordi &p1, const uint64_t width, int layer = 10000, bool tr = true);
+    virtual void img_arc(const Arc &arc);
+    virtual bool img_supports_arc() const
+    {
+        return false;
+    }
     virtual void img_hole(const Hole &hole)
     {
     }
+    virtual void img_text(const Text *text)
+    {
+    }
     virtual void img_patch_type(PatchType type)
     {
     }
@@ -264,6 +269,6 @@ private:
 
     void draw_direction(Pin::Direction dir, ColorP color);
 
-    TextData::Buffer text_data_buffer;
+    CanvasTextRenderer text_renderer;
 };
 } // namespace horizon
