@@ -64,6 +64,32 @@ CanvasGL::CanvasGL()
     gesture_drag->signal_update().connect(sigc::mem_fun(*this, &CanvasGL::drag_gesture_update_cb));
     gesture_drag->set_propagation_phase(Gtk::PHASE_BUBBLE);
     gesture_drag->set_touch_only(true);
+
+    signal_query_tooltip().connect(
+            [this](int x, int y, bool keyboard_tooltip, const Glib::RefPtr<Gtk::Tooltip> &tooltip) {
+                if (keyboard_tooltip)
+                    return false;
+                const auto mkrs = markers.get_markers_at_screen_pos(x, y);
+                const size_t max_lines = 5;
+                const size_t max_markers = (mkrs.size() > max_lines) ? max_lines - 1 : max_lines;
+                std::string l;
+                for (size_t i = 0; i < std::min(mkrs.size(), max_markers); i++) {
+                    const auto &m = *mkrs.at(i);
+                    if (m.label.size()) {
+                        if (l.size())
+                            l += "\n";
+                        l += m.label;
+                    }
+                }
+                if (mkrs.size() > max_lines) {
+                    l += "\n… " + std::to_string(mkrs.size() - max_markers) + " more";
+                }
+                if (l.size()) {
+                    tooltip->set_text(l);
+                    return true;
+                }
+                return false;
+            });
 }
 
 void CanvasGL::set_grid_spacing(uint64_t x, uint64_t y)
@@ -232,13 +258,15 @@ bool CanvasGL::on_render(const Glib::RefPtr<Gdk::GLContext> &context)
     GL_CHECK_ERROR
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-    picture_renderer.render(false);
+    if (show_pictures)
+        picture_renderer.render(false);
     GL_CHECK_ERROR
     grid.render();
     GL_CHECK_ERROR
     triangle_renderer.render();
     GL_CHECK_ERROR
-    picture_renderer.render(true);
+    if (show_pictures)
+        picture_renderer.render(true);
     GL_CHECK_ERROR
     selectables_renderer.render();
     GL_CHECK_ERROR
@@ -416,8 +444,10 @@ void CanvasGL::request_push(PushFilter filter)
 void CanvasGL::center_and_zoom(const Coordf &center, float sc)
 {
     // we want center to be at width, height/2
-    if (sc > 0)
+    if (sc > 0) {
         scale = sc;
+        update_viewmat();
+    }
     const Coordf m(m_width / 2, m_height / 2);
     const auto c = canvas2screen(center);
     offset -= (c - m);
@@ -445,9 +475,6 @@ void CanvasGL::zoom_to_bbox(const std::pair<Coordf, Coordf> &bb)
 
 void CanvasGL::zoom_to(const Coordf &c, float inc)
 {
-    if (!can_set_scale())
-        return;
-
     if (smooth_zoom)
         start_smooth_zoom(c, inc);
     else
@@ -815,6 +842,10 @@ void CanvasGL::set_colors2(const std::vector<ColorI> &c)
     request_push();
 }
 
+void CanvasGL::set_msd_params(const MSD::Params &p)
+{
+    zoom_animator.set_params(p);
+}
 
 static const float char_space = 1;
 
