@@ -192,6 +192,10 @@ public:
 
     void append_target(const Target &target);
 
+    void set_msd_params(const MSD::Params &params);
+
+    bool show_pictures = true;
+
 protected:
     void push() override;
     void request_push() override;
