@@ -31,6 +31,7 @@
 #include "util/polygon_arc_removal_proxy.hpp"
 #include <algorithm>
 #include <ctime>
+#include <iostream>
 #include <sstream>
 
 namespace horizon {
@@ -69,7 +70,8 @@ void Canvas::render(const SchematicJunction &junc)
     }
     object_ref_push(ObjectType::JUNCTION, junc.uuid);
 
-    auto connection_count = junc.connected_net_lines.size() + junc.connected_power_symbols.size();
+    auto connection_count =
+            junc.connected_net_lines.size() + junc.connected_power_symbols.size() + junc.connected_net_ties.size();
     if (connection_count == 2) {
         if (show_all_junctions_in_schematic)
             draw_plus(junc.position, 250000, c);
@@ -79,7 +81,7 @@ void Canvas::render(const SchematicJunction &junc)
         img_line(junc.position, junc.position + Coordi(0, 1000), 0.75_mm, 0, true);
     }
     else if (junc.connected_bus_labels.size() || junc.connected_bus_rippers.size() || junc.connected_net_labels.size()
-             || junc.connected_lines.size() || junc.connected_arcs.size()) {
+             || junc.connected_lines.size() || junc.connected_arcs.size() || junc.connected_net_ties.size()) {
         // nop
     }
     else {
@@ -253,6 +255,61 @@ void Canvas::render(const LineNet &line)
     selectables.append_line(line.uuid, ObjectType::LINE_NET, line.from.get_position(), line.to.get_position(), width);
 }
 
+static std::string get_name(const std::string &n)
+{
+    if (n.size())
+        return n;
+    else
+        return "unnamed net";
+}
+
+void Canvas::render(const SchematicNetTie &tie)
+{
+    const auto from = tie.from->position;
+    const auto to = tie.to->position;
+
+    object_ref_push(ObjectType::SCHEMATIC_NET_TIE, tie.uuid);
+    // draw_line(from, to, ColorP::NET_TIE, 0, true, width);
+    Coordf p0;
+    Coordf p1;
+    if (from < to) {
+        p0 = from;
+        p1 = to;
+    }
+    else {
+        p0 = to;
+        p1 = from;
+    }
+    const auto v = p1 - p0;
+    const auto vn = v.normalize();
+    const auto center = ((p0 + p1) / 2);
+    const auto perp_n = Coordf(v.y, -v.x).normalize();
+    const float h = .5_mm;
+    const auto s = v.mag();
+    const auto r = (4 * h * h + s * s) / (8 * h);
+    const auto d = r - h;
+    const auto arc_center = center + (perp_n * d);
+    const float a0 = c2pi(atan2f(p0.y - arc_center.y, p0.x - arc_center.x));
+    const float a1 = c2pi(atan2f(p1.y - arc_center.y, p1.x - arc_center.x));
+    // img_auto_line = img_mode;
+    draw_arc2(arc_center, r, a1, a0, ColorP::NET_TIE, 0, 0);
+    draw_arc2(center - (perp_n * d), r, a1 + M_PI, a0 + M_PI, ColorP::NET_TIE, 0, 0);
+
+
+    const auto text_pos = center + (perp_n * 1.5_mm);
+
+    TextRenderer::Options opts;
+    opts.center = true;
+    draw_text(text_pos, 1.5_mm,
+              get_name(tie.net_tie->net_primary->name) + "\n" + get_name(tie.net_tie->net_secondary->name),
+              angle_from_rad(atan2(v.y, v.x)), TextOrigin::CENTER, ColorP::NET_TIE, 0, opts);
+    // img_auto_line = false;
+    object_ref_pop();
+    if (img_mode)
+        return;
+    selectables.append_line(tie.uuid, ObjectType::SCHEMATIC_NET_TIE, p0 + (vn * h), p1 - (vn * h), h * 2);
+}
+
 void Canvas::render(const Track &track, bool interactive)
 {
     auto c = ColorP::FROM_LAYER;
@@ -260,11 +317,15 @@ void Canvas::render(const Track &track, bool interactive)
         c = ColorP::BUS;
     }
     auto width = track.width;
+    if (interactive)
+        object_ref_push(ObjectType::TRACK, track.uuid);
     img_net(track.net);
     img_patch_type(PatchType::TRACK);
     img_line(track.from.get_position(), track.to.get_position(), width, track.layer);
     img_patch_type(PatchType::OTHER);
     img_net(nullptr);
+    if (interactive)
+        object_ref_pop();
     if (img_mode)
         return;
     auto layer = track.layer;
@@ -303,6 +364,55 @@ void Canvas::render(const Track &track, bool interactive)
                                 track.width, 0, track.layer);
 }
 
+void Canvas::render(const BoardNetTie &tie, bool interactive)
+{
+    auto c = ColorP::FROM_LAYER;
+    auto width = tie.width;
+
+
+    auto layer = tie.layer;
+
+    auto center = (tie.from->position + tie.to->position) / 2;
+    if (interactive)
+        object_ref_push(ObjectType::BOARD_NET_TIE, tie.uuid);
+
+    if (img_mode) {
+        img_net(tie.net_tie->net_primary);
+        img_patch_type(PatchType::NET_TIE);
+        img_line(tie.from->position, tie.to->position, width, tie.layer);
+        img_patch_type(PatchType::OTHER);
+        img_net(nullptr);
+    }
+
+    draw_line(tie.from->position, tie.to->position, c, layer, true, width);
+    if (interactive && show_text_in_tracks && width > 0) {
+        auto overlay_layer = get_overlay_layer(tie.layer, true);
+        set_lod_size(width);
+        const auto vec = (tie.from->position - tie.to->position);
+        const auto length = vec.magd();
+        Placement p;
+        p.set_angle_rad(get_view_angle());
+        if (get_flip_view())
+            p.invert_angle();
+        p.accumulate(Placement(center));
+        p.set_angle_rad(p.get_angle_rad() + atan2(vec.y, vec.x));
+        if (get_flip_view()) {
+            p.shift.x *= -1;
+            p.invert_angle();
+        }
+        draw_bitmap_text_box(p, length, width, tie.net_tie->net_primary->name + "<>" + tie.net_tie->net_secondary->name,
+                             ColorP::TEXT_OVERLAY, overlay_layer, TextBoxMode::FULL);
+        set_lod_size(-1);
+    }
+    if (interactive)
+        object_ref_pop();
+    if (img_mode)
+        return;
+    if (interactive)
+        selectables.append_line(tie.uuid, ObjectType::BOARD_NET_TIE, tie.from->position, tie.to->position, tie.width, 0,
+                                tie.layer);
+}
+
 void Canvas::render(const ConnectionLine &line)
 {
     if (img_mode)
@@ -312,8 +422,9 @@ void Canvas::render(const ConnectionLine &line)
                             0);
 }
 
-void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
+void Canvas::render(const SymbolPin &pin, SymbolMode mode, ColorP co)
 {
+    const bool interactive = mode != SymbolMode::SHEET;
     Coordi p0 = transform.transform(pin.position);
     Coordi p1 = p0;
 
@@ -416,7 +527,7 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
         c_pin = co;
     }
     img_auto_line = img_mode;
-    if (interactive || pin.name_visible) {
+    if (mode == SymbolMode::EDIT || pin.name_visible) {
         bool draw_in_line = pin.name_orientation == SymbolPin::NameOrientation::IN_LINE
                             || (pin.name_orientation == SymbolPin::NameOrientation::HORIZONTAL
                                 && (pin_orientation == Orientation::LEFT || pin_orientation == Orientation::RIGHT));
@@ -429,17 +540,25 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
             Placement tr;
             tr.set_angle(orientation_to_angle(pin_orientation));
             auto shift = tr.transform(Coordi(-1_mm, 0));
-            TextOptions opts;
+            TextRenderer::Options opts;
             opts.center = true;
             draw_text(p_name + shift, 1.5_mm, pin.name, orientation_to_angle(name_orientation) + 16384,
                       TextOrigin::CENTER, c_name, 0, opts);
         }
     }
-    std::pair<Coordf, Coordf> pad_extents;
-    if (interactive || pin.pad_visible) {
+    std::optional<std::pair<Coordf, Coordf>> pad_extents;
+    if (mode == SymbolMode::EDIT || pin.pad_visible) {
         pad_extents = draw_text(p_pad, 0.75_mm, pin.pad, orientation_to_angle(pad_orientation), TextOrigin::BASELINE,
                                 c_pad, 0, {});
     }
+    if (interactive) {
+        if (pad_extents.has_value())
+            selectables.append(pin.uuid, ObjectType::SYMBOL_PIN, p0,
+                               Coordf::min(pad_extents->first, Coordf::min(p0, p1)),
+                               Coordf::max(pad_extents->second, Coordf::max(p0, p1)));
+        else
+            selectables.append_line(pin.uuid, ObjectType::SYMBOL_PIN, p0, p1, 0);
+    }
 
     transform_save();
     transform.accumulate(pin.position);
@@ -549,9 +668,6 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
         draw_line(p0, p0 + Coordi(0, 10), c_main, 0, false, 0.75_mm);
     }
     draw_line(p0, p1, c_main, 0, false);
-    if (interactive)
-        selectables.append(pin.uuid, ObjectType::SYMBOL_PIN, p0, Coordf::min(pad_extents.first, Coordf::min(p0, p1)),
-                           Coordf::max(pad_extents.second, Coordf::max(p0, p1)));
     img_auto_line = false;
 }
 
@@ -564,13 +680,17 @@ void Canvas::render(const Arc &arc, bool interactive, ColorP co)
     const float a0 = c2pi(atan2f(a.y - c.y, a.x - c.x));
     const float a1 = c2pi(atan2f(b.y - c.y, b.x - c.x));
     const float dphi = c2pi(a1 - a0);
-
-    draw_arc2(c, radius0, a0, a1, co, arc.layer, arc.width);
-    Coordf t(radius0, radius0);
-    if (interactive) {
-        const float ax = std::min(asin(arc.width / 2 / radius0), static_cast<float>((2 * M_PI - dphi) / 2 - .1e-4));
-        selectables.append_arc(arc.uuid, ObjectType::ARC, c, radius0 - arc.width / 2, radius0 + arc.width / 2, a0 - ax,
-                               a1 + ax, 0, arc.layer);
+    if (img_mode && img_supports_arc()) {
+        img_arc(arc);
+    }
+    else {
+        draw_arc2(c, radius0, a0, a1, co, arc.layer, arc.width);
+        Coordf t(radius0, radius0);
+        if (interactive) {
+            const float ax = std::min(asin(arc.width / 2 / radius0), static_cast<float>((2 * M_PI - dphi) / 2 - .1e-4));
+            selectables.append_arc(arc.uuid, ObjectType::ARC, c, radius0 - arc.width / 2, radius0 + arc.width / 2,
+                                   a0 - ax, a1 + ax, 0, arc.layer);
+        }
     }
 }
 
@@ -578,7 +698,7 @@ void Canvas::render(const SchematicSymbol &sym)
 {
     transform = sym.placement;
     object_ref_push(ObjectType::SCHEMATIC_SYMBOL, sym.uuid);
-    render(sym.symbol, true, sym.smashed, ColorP::FROM_LAYER);
+    render(sym.symbol, SymbolMode::SHEET, sym.smashed, ColorP::FROM_LAYER);
     object_ref_pop();
     for (const auto &it : sym.symbol.pins) {
         targets.emplace_back(UUIDPath<2>(sym.uuid, it.second.uuid), ObjectType::SYMBOL_PIN,
@@ -607,33 +727,15 @@ void Canvas::render(const SchematicSymbol &sym)
 
 void Canvas::render(const Text &text, bool interactive, ColorP co)
 {
-    bool rev = layer_provider->get_layers().at(text.layer).reverse;
-    transform_save();
-    const auto mirror_orig = transform.mirror;
-    const auto angle_orig = transform.get_angle();
-    transform.accumulate(text.placement);
-    const auto text_angle = text.placement.get_angle();
-    const auto angle =
-            (transform.mirror ^ rev ? 32768 - text_angle : text_angle) + (mirror_orig ^ rev ? -1 : 1) * angle_orig;
-
+    const bool rev = layer_provider->get_layers().at(text.layer).reverse;
     img_patch_type(PatchType::TEXT);
     triangle_type_current = TriangleInfo::Type::TEXT;
-    object_ref_push(ObjectType::TEXT, text.uuid);
-
-    TextOptions opts;
-    opts.flip = rev;
-    opts.mirror = transform.mirror;
-    opts.font = text.font;
-    opts.width = text.width;
-    opts.allow_upside_down = text.allow_upside_down;
-
-    const auto extents = draw_text(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle,
-                                   text.origin, co, text.layer, opts);
+    img_text(&text);
+    const auto extents = text_renderer.render(text, co, transform, rev);
+    img_text(nullptr);
     triangle_type_current = TriangleInfo::Type::NONE;
-    object_ref_pop();
-    // img_text_extents(text, extents);
     img_patch_type(PatchType::OTHER);
-    transform_restore();
+
     if (interactive) {
         selectables.append(text.uuid, ObjectType::TEXT, text.placement.shift, extents.first, extents.second, 0,
                            text.layer);
@@ -774,22 +876,6 @@ static const Coordf coordf_from_pt(const TPPLPoint &p)
     return Coordf(p.x, p.y);
 }
 
-static bool poly_is_rect(const Polygon &poly)
-{
-    if (poly.vertices.size() != 4)
-        return false;
-    for (size_t i = 0; i < 4; i++) {
-        const auto &p0 = poly.get_vertex(i).position;
-        const auto &p1 = poly.get_vertex(i + 1).position;
-        const auto &p2 = poly.get_vertex(i + 2).position;
-        const auto v0 = p1 - p0;
-        const auto v1 = p2 - p1;
-        if (v0.dot(v1) != 0)
-            return false;
-    }
-    return true;
-}
-
 void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
 {
     img_polygon(ipoly);
@@ -838,7 +924,7 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
     else { // normal polygon
         const bool is_keepout = dynamic_cast<Keepout *>(poly.usage.ptr);
         begin_group(poly.layer);
-        if (poly_is_rect(poly) && !is_keepout) {
+        if (poly.is_rect() && !is_keepout) {
             const Coordf p0 = (poly.get_vertex(0).position + poly.get_vertex(1).position) / 2;
             const Coordf p1 = (poly.get_vertex(2).position + poly.get_vertex(3).position) / 2;
             const float width = (poly.get_vertex(0).position - poly.get_vertex(1).position).magd();
@@ -1015,12 +1101,13 @@ void Canvas::render(const Pad &pad)
     transform_restore();
 }
 
-void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed, ColorP co)
+void Canvas::render(const Symbol &sym, SymbolMode mode, bool smashed, ColorP co)
 {
+    const bool on_sheet = mode == SymbolMode::SHEET;
     if (!on_sheet) {
         for (const auto &it : sym.junctions) {
             auto &junc = it.second;
-            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, true);
+            selectables.append(junc.uuid, ObjectType::JUNCTION, junc.position, 0, 10000, mode == SymbolMode::EDIT);
             targets.emplace_back(junc.uuid, ObjectType::JUNCTION, transform.transform(junc.position));
         }
     }
@@ -1033,13 +1120,13 @@ void Canvas::render(const Symbol &sym, bool on_sheet, bool smashed, ColorP co)
         auto sym_uuid = object_refs_current.back().uuid;
         for (const auto &it : sym.pins) {
             object_ref_push(ObjectType::SYMBOL_PIN, it.second.uuid, sym_uuid);
-            render(it.second, !on_sheet, co);
+            render(it.second, mode, co);
             object_ref_pop();
         }
     }
     else {
         for (const auto &it : sym.pins) {
-            render(it.second, !on_sheet, co);
+            render(it.second, mode, co);
         }
     }
 
@@ -1101,13 +1188,13 @@ void Canvas::render(const Sheet &sheet)
     for (const auto &it : sheet.block_symbols) {
         render(it.second);
     }
+    for (const auto &it : sheet.net_ties) {
+        render(it.second);
+    }
 }
 
 void Canvas::render(const Padstack &padstack, bool interactive)
 {
-    for (const auto &it : padstack.holes) {
-        render(it.second, interactive);
-    }
     img_padstack(padstack);
     img_set_padstack(true);
     for (const auto &it : padstack.polygons) {
@@ -1117,6 +1204,9 @@ void Canvas::render(const Padstack &padstack, bool interactive)
         render(it.second, interactive);
     }
     img_set_padstack(false);
+    for (const auto &it : padstack.holes) {
+        render(it.second, interactive);
+    }
 }
 
 void Canvas::render_pad_overlay(const Pad &pad, bool interactive)
@@ -1425,7 +1515,7 @@ void Canvas::render(const class Dimension &dim)
         auto length = v.mag();
         auto s = dim_to_string(length, false);
 
-        TextOptions opts;
+        TextRenderer::Options opts;
         opts.draw = false;
         const auto text_bb = draw_text({0, 0}, dim.label_size, s, 0, TextOrigin::CENTER, co, 10000, opts);
         auto text_width = std::abs(text_bb.second.x - text_bb.first.x);
@@ -1463,7 +1553,7 @@ void Canvas::render(const class Dimension &dim)
         transform_restore();
 
         const int angle_i = angle_from_rad(angle);
-        TextOptions opts2;
+        TextRenderer::Options opts2;
         opts2.flip = get_flip_view();
         const auto real_text_bb =
                 draw_text(Coordi(text_pos.x, text_pos.y), dim.label_size, s,
@@ -1519,6 +1609,9 @@ void Canvas::render(const Board &brd, bool interactive, PanelMode mode, OutlineM
     for (const auto &it : brd.decals) {
         render(it.second);
     }
+    for (const auto &it : brd.net_ties) {
+        render(it.second, interactive);
+    }
     if (mode == PanelMode::INCLUDE) {
         for (const auto &it : brd.board_panels) {
             render(it.second);
@@ -1797,7 +1890,7 @@ void Canvas::render(const class BlockSymbolPort &port, bool interactive)
         Placement tr;
         tr.set_angle(orientation_to_angle(port_orientation));
         auto shift = tr.transform(Coordi(-1_mm, 0));
-        TextOptions opts;
+        TextRenderer::Options opts;
         opts.center = true;
         draw_text(p_name + shift, 1.5_mm, port.name, orientation_to_angle(name_orientation) + 16384, TextOrigin::CENTER,
                   c_name, 0, opts);
