@@ -22,6 +22,16 @@ Canvas3DBase::Canvas3DBase()
 {
 }
 
+const std::map<int, CanvasMesh::Layer3D> &Canvas3DBase::get_layers() const
+{
+    return ca.get_layers();
+}
+
+const CanvasMesh::Layer3D &Canvas3DBase::get_layer(int layer) const
+{
+    return get_layers().at(layer);
+}
+
 Color Canvas3DBase::get_layer_color(int layer) const
 {
     if (layer == 20000 || BoardLayers::is_copper(layer)) { // pth or cu
@@ -51,19 +61,19 @@ float Canvas3DBase::get_layer_offset(int layer) const
         return get_layer_offset(BoardLayers::TOP_COPPER);
 
     else
-        return ca.get_layer(layer).offset + ca.get_layer(layer).explode_mul * explode;
+        return get_layer(layer).offset + get_layer(layer).explode_mul * explode;
 }
 
 float Canvas3DBase::get_layer_thickness(int layer) const
 {
     if (layer == BoardLayers::L_OUTLINE && explode == 0) {
-        return ca.get_layer(BoardLayers::BOTTOM_COPPER).offset + ca.get_layer(BoardLayers::BOTTOM_COPPER).thickness;
+        return get_layer(BoardLayers::BOTTOM_COPPER).offset + get_layer(BoardLayers::BOTTOM_COPPER).thickness;
     }
     else if (layer == 20000) {
         return -(get_layer_offset(BoardLayers::TOP_COPPER) - get_layer_offset(BoardLayers::BOTTOM_COPPER));
     }
     else {
-        return ca.get_layer(layer).thickness;
+        return get_layer(layer).thickness;
     }
 }
 
@@ -332,51 +342,50 @@ void Canvas3DBase::render(RenderBackground mode)
     glFlush();
 }
 
-void Canvas3DBase::view_all()
+std::optional<Canvas3DBase::ViewParams> Canvas3DBase::get_view_all_params() const
 {
     if (!brd)
-        return;
+        return {};
+    ViewParams r;
 
-    const auto &vertices = ca.get_layer(BoardLayers::L_OUTLINE).walls;
-    MinMaxAccumulator<float> acc_x, acc_y;
+    auto bb = ca.get_bbox();
 
-    for (const auto &it : vertices) {
-        acc_x.accumulate(it.x);
-        acc_y.accumulate(it.y);
-    }
-
-    float xmin = acc_x.get_min();
-    float xmax = acc_x.get_max();
-    float ymin = acc_y.get_min();
-    float ymax = acc_y.get_max();
+    float xmin = bb.first.x;
+    float xmax = bb.second.x;
+    float ymin = bb.first.y;
+    float ymax = bb.second.y;
 
     float board_width = (xmax - xmin) / 1e6;
     float board_height = (ymax - ymin) / 1e6;
 
     if (board_height < 1 || board_width < 1)
-        return;
+        return {};
 
-    set_center({(xmin + xmax) / 2e6, (ymin + ymax) / 2e6});
+    r.cx = (xmin + xmax) / 2e6;
+    r.cy = (ymin + ymax) / 2e6;
 
+    r.cam_distance = std::max(board_width / width, board_height / height) / (2 * get_magic_number() / height) * 1.1;
+    r.cam_azimuth = 270;
+    r.cam_elevation = 89.99;
 
-    set_cam_distance(std::max(board_width / width, board_height / height) / (2 * get_magic_number() / height) * 1.1);
-    set_cam_azimuth(270);
-    set_cam_elevation(89.99);
+    return r;
+}
+
+void Canvas3DBase::view_all()
+{
+    if (const auto p = get_view_all_params()) {
+        set_center({p->cx, p->cy});
+        set_cam_distance(p->cam_distance);
+        set_cam_azimuth(p->cam_azimuth);
+        set_cam_elevation(p->cam_elevation);
+    }
 }
 
 void Canvas3DBase::prepare()
 {
-    bbox.first = glm::vec3();
-    bbox.second = glm::vec3();
-    for (const auto &it : ca.get_patches()) {
-        for (const auto &path : it.second) {
-            for (const auto &p : path) {
-                glm::vec3 q(p.X / 1e6, p.Y / 1e6, 0);
-                bbox.first = glm::min(bbox.first, q);
-                bbox.second = glm::max(bbox.second, q);
-            }
-        }
-    }
+    const auto bb = ca.get_bbox();
+    bbox.first = glm::vec3(bb.first.x / 1e6, bb.first.y / 1e6, 0);
+    bbox.second = glm::vec3(bb.second.x / 1e6, bb.second.y / 1e6, 0);
 }
 
 int Canvas3DBase::a_get_scale_factor() const
