@@ -10,6 +10,7 @@
 #include <glibmm/ustring.h>
 #include <glibmm/fileutils.h>
 #include "core/tool_id.hpp"
+#include <iostream>
 
 namespace horizon {
 
@@ -159,7 +160,10 @@ void Core::rebuild_finish(bool from_undo, const std::string &comment)
             history.pop_back();
         }
         assert(history_current + 1 == (int)history.size());
-        history_push(comment);
+        if (const auto comment_ctx = get_history_comment_context(); comment_ctx.size())
+            history_push(comment + " " + comment_ctx);
+        else
+            history_push(comment);
         history_current++;
         history_trim();
     }
@@ -167,6 +171,11 @@ void Core::rebuild_finish(bool from_undo, const std::string &comment)
     signal_can_undo_redo().emit();
 }
 
+std::string Core::get_history_comment_context() const
+{
+    return {};
+}
+
 void Core::undo()
 {
     if (history_current) {
