@@ -10,6 +10,7 @@
 #include "util/picture_load.hpp"
 #include "blocks/iblock_provider.hpp"
 #include "blocks/blocks.hpp"
+#include <iostream>
 
 namespace horizon {
 
@@ -47,10 +48,10 @@ CoreBoard::CoreBoard(const std::string &board_filename, const std::string &block
                      const std::string &pictures_dir, IPool &pool, IPool &pool_caching)
     : Core(pool, &pool_caching), block(get_flattend_block(blocks_filename, pool_caching)),
       brd(Board::new_from_file(board_filename, *block, pool_caching)), rules(brd->rules),
-      fab_output_settings(brd->fab_output_settings), pdf_export_settings(brd->pdf_export_settings),
-      step_export_settings(brd->step_export_settings), pnp_export_settings(brd->pnp_export_settings),
-      grid_settings(brd->grid_settings), colors(brd->colors), m_board_filename(board_filename),
-      m_blocks_filename(blocks_filename), m_pictures_dir(pictures_dir)
+      gerber_output_settings(brd->gerber_output_settings), odb_output_settings(brd->odb_output_settings),
+      pdf_export_settings(brd->pdf_export_settings), step_export_settings(brd->step_export_settings),
+      pnp_export_settings(brd->pnp_export_settings), grid_settings(brd->grid_settings), colors(brd->colors),
+      m_board_filename(board_filename), m_blocks_filename(blocks_filename), m_pictures_dir(pictures_dir)
 {
     brd->load_pictures(pictures_dir);
     rebuild("init");
@@ -73,6 +74,15 @@ void CoreBoard::reload_netlist()
         }
     }
 
+    for (auto it = brd->net_ties.begin(); it != brd->net_ties.end();) {
+        if (it->second.net_tie == nullptr) {
+            brd->net_ties.erase(it++);
+        }
+        else {
+            it++;
+        }
+    }
+
     // delete planes with deleted nets
     for (auto it = brd->planes.begin(); it != brd->planes.end();) {
         const auto &plane = it->second;
@@ -226,6 +236,32 @@ bool CoreBoard::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID
         }
     } break;
 
+    case ObjectType::BOARD_NET_TIE: {
+        const auto &tie = brd->net_ties.at(uu);
+        switch (property) {
+        case ObjectProperty::ID::WIDTH_FROM_RULES:
+            dynamic_cast<PropertyValueBool &>(value).value = tie.width_from_rules;
+            return true;
+
+        case ObjectProperty::ID::WIDTH:
+            dynamic_cast<PropertyValueInt &>(value).value = tie.width;
+            return true;
+
+        case ObjectProperty::ID::LAYER:
+            dynamic_cast<PropertyValueInt &>(value).value = tie.layer;
+            return true;
+
+        case ObjectProperty::ID::NAME:
+            dynamic_cast<PropertyValueString &>(value).value =
+                    tie.net_tie->net_primary->name + "\n" + tie.net_tie->net_secondary->name;
+            return true;
+
+
+        default:
+            return false;
+        }
+    } break;
+
     case ObjectType::VIA: {
         auto via = &brd->vias.at(uu);
         switch (property) {
@@ -480,6 +516,28 @@ bool CoreBoard::set_property(ObjectType type, const UUID &uu, ObjectProperty::ID
         }
     } break;
 
+    case ObjectType::BOARD_NET_TIE: {
+        auto &tie = brd->net_ties.at(uu);
+        switch (property) {
+        case ObjectProperty::ID::WIDTH_FROM_RULES:
+            tie.width_from_rules = dynamic_cast<const PropertyValueBool &>(value).value;
+            break;
+
+        case ObjectProperty::ID::WIDTH:
+            if (tie.width_from_rules)
+                return false;
+            tie.width = dynamic_cast<const PropertyValueInt &>(value).value;
+            break;
+
+        case ObjectProperty::ID::LAYER:
+            tie.layer = dynamic_cast<const PropertyValueInt &>(value).value;
+            break;
+
+        default:
+            return false;
+        }
+    } break;
+
     case ObjectType::VIA: {
         auto via = &brd->vias.at(uu);
         switch (property) {
@@ -618,6 +676,23 @@ bool CoreBoard::get_property_meta(ObjectType type, const UUID &uu, ObjectPropert
         }
     } break;
 
+    case ObjectType::BOARD_NET_TIE: {
+        auto &tie = brd->net_ties.at(uu);
+        switch (property) {
+        case ObjectProperty::ID::WIDTH:
+            meta.is_settable = !tie.width_from_rules;
+            return true;
+
+        case ObjectProperty::ID::LAYER:
+            layers_to_meta(meta);
+            return true;
+
+        default:
+            return false;
+        }
+    } break;
+
+
     case ObjectType::PLANE: {
         auto plane = &brd->planes.at(uu);
         switch (property) {
@@ -735,7 +810,8 @@ const std::string &CoreBoard::get_filename() const
 void CoreBoard::save(const std::string &suffix)
 {
     brd->rules = rules;
-    brd->fab_output_settings = fab_output_settings;
+    brd->gerber_output_settings = gerber_output_settings;
+    brd->odb_output_settings = odb_output_settings;
     brd->pdf_export_settings = pdf_export_settings;
     brd->step_export_settings = step_export_settings;
     brd->pnp_export_settings = pnp_export_settings;
