@@ -11,6 +11,7 @@
 #include "util/picture_load.hpp"
 #include "logger/logger.hpp"
 #include <filesystem>
+#include <iostream>
 
 namespace horizon {
 namespace fs = std::filesystem;
@@ -66,11 +67,21 @@ Block *CoreSchematic::get_current_block()
     return &blocks->blocks.at(block_uuid).block;
 }
 
+const Block *CoreSchematic::get_current_block() const
+{
+    return &blocks->blocks.at(block_uuid).block;
+}
+
 Block *CoreSchematic::get_top_block()
 {
     return &blocks->get_top_block_item().block;
 }
 
+const Block *CoreSchematic::get_top_block() const
+{
+    return &blocks->get_top_block_item().block;
+}
+
 LayerProvider &CoreSchematic::get_layer_provider()
 {
     if (get_block_symbol_mode())
@@ -178,6 +189,7 @@ bool CoreSchematic::has_object_type(ObjectType ty) const
         case ObjectType::LINE:
         case ObjectType::ARC:
         case ObjectType::PICTURE:
+        case ObjectType::SCHEMATIC_NET_TIE:
             return true;
             break;
         default:
@@ -947,6 +959,22 @@ void CoreSchematic::history_load(unsigned int i)
     s_signal_rebuilt.emit();
 }
 
+std::string CoreSchematic::get_history_comment_context() const
+{
+    if (get_block_symbol_mode()) {
+        return "on block symbol " + get_current_block()->name;
+    }
+    else if (instance_path.size()) {
+        return "on sheet " + get_top_block()->instance_path_to_string(instance_path) + "/" + get_sheet()->name;
+    }
+    else if (current_block_is_top()) {
+        return "on sheet " + get_sheet()->name;
+    }
+    else {
+        return "on sheet " + get_current_block()->name + "/" + get_sheet()->name;
+    }
+}
+
 void CoreSchematic::reload_pool()
 {
 
