@@ -31,7 +31,9 @@ public:
     void delete_junction(const UUID &uu) override;
 
     class Block *get_top_block() override;
+    const Block *get_top_block() const;
     class Block *get_current_block() override;
+    const Block *get_current_block() const;
     class LayerProvider &get_layer_provider() override;
 
     bool set_property(ObjectType type, const UUID &uu, ObjectProperty::ID property,
@@ -132,6 +134,7 @@ private:
     void rebuild_internal(bool from_undo, const std::string &comment) override;
     void history_push(const std::string &comment) override;
     void history_load(unsigned int i) override;
+    std::string get_history_comment_context() const override;
     void save(const std::string &suffix) override;
     void delete_autosave() override;
     void fix_current_block();
