@@ -46,6 +46,7 @@
 #include "tools/tool_place_via.hpp"
 #include "tools/tool_rotate_arbitrary.hpp"
 #include "tools/tool_route_track_interactive.hpp"
+#include "tools/tool_select_connected_lines.hpp"
 #include "tools/tool_set_diffpair.hpp"
 #include "tools/tool_set_via_net.hpp"
 #include "tools/tool_smash.hpp"
@@ -57,6 +58,7 @@
 #include "tools/tool_copy_tracks.hpp"
 #include "tools/tool_swap_nets.hpp"
 #include "tools/tool_line_loop_to_polygon.hpp"
+#include "tools/tool_lines_to_tracks.hpp"
 #include "tools/tool_change_unit.hpp"
 #include "tools/tool_set_nc_all.hpp"
 #include "tools/tool_set_nc.hpp"
@@ -90,6 +92,13 @@
 #include "tools/tool_map_port.hpp"
 #include "tools/tool_add_block_instance.hpp"
 #include "tools/tool_align_and_distribute.hpp"
+#include "tools/tool_merge_duplicate_junctions.hpp"
+#include "tools/tool_manage_power_nets.hpp"
+#include "tools/tool_edit_text.hpp"
+#include "tools/tool_tie_nets.hpp"
+#include "tools/tool_draw_net_tie.hpp"
+#include "tools/tool_flip_net_tie.hpp"
+#include "tools/tool_move_track_connection.hpp"
 
 namespace horizon {
 
@@ -174,13 +183,16 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::EDIT_STACKUP:
     case ToolID::ANNOTATE:
     case ToolID::EDIT_SCHEMATIC_PROPERTIES:
-    case ToolID::MANAGE_POWER_NETS:
+    case ToolID::EDIT_PROJECT_PROPERTIES:
     case ToolID::EDIT_FRAME_PROPERTIES:
     case ToolID::TOGGLE_GROUP_TAG_VISIBLE:
     case ToolID::MANAGE_INCLUDED_BOARDS:
     case ToolID::MANAGE_PORTS:
         return std::make_unique<ToolManageBuses>(this, tool_id);
 
+    case ToolID::MANAGE_POWER_NETS:
+        return std::make_unique<ToolManagePowerNets>(this, tool_id);
+
     case ToolID::DRAW_POLYGON:
         return std::make_unique<ToolDrawPolygon>(this, tool_id);
 
@@ -271,6 +283,9 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::DRAW_DIMENSION:
         return std::make_unique<ToolDrawDimension>(this, tool_id);
 
+    case ToolID::SELECT_CONNECTED_LINES:
+        return std::make_unique<ToolSelectConnectedLines>(this, tool_id);
+
     case ToolID::SET_DIFFPAIR:
     case ToolID::CLEAR_DIFFPAIR:
         return std::make_unique<ToolSetDiffpair>(this, tool_id);
@@ -321,6 +336,9 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::LINE_LOOP_TO_POLYGON:
         return std::make_unique<ToolLineLoopToPolygon>(this, tool_id);
 
+    case ToolID::LINES_TO_TRACKS:
+        return std::make_unique<ToolLinesToTracks>(this, tool_id);
+
     case ToolID::CHANGE_UNIT:
         return std::make_unique<ToolChangeUnit>(this, tool_id);
 
@@ -426,6 +444,24 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::ALIGN_AND_DISTRIBUTE:
         return std::make_unique<ToolAlignAndDistribute>(this, tool_id);
 
+    case ToolID::MERGE_DUPLICATE_JUNCTIONS:
+        return std::make_unique<ToolMergeDuplicateJunctions>(this, tool_id);
+
+    case ToolID::EDIT_TEXT:
+        return std::make_unique<ToolEditText>(this, tool_id);
+
+    case ToolID::TIE_NETS:
+        return std::make_unique<ToolTieNets>(this, tool_id);
+
+    case ToolID::DRAW_NET_TIE:
+        return std::make_unique<ToolDrawNetTie>(this, tool_id);
+
+    case ToolID::FLIP_NET_TIE:
+        return std::make_unique<ToolFlipNetTie>(this, tool_id);
+
+    case ToolID::MOVE_TRACK_CONNECTION:
+        return std::make_unique<ToolMoveTrackConnection>(this, tool_id);
+
     default:
         return nullptr;
     }
