@@ -23,6 +23,7 @@ enum class ToolID {
     PLACE_NET_LABEL,
     DISCONNECT,
     BEND_LINE_NET,
+    SELECT_CONNECTED_LINES,
     SELECT_NET_SEGMENT,
     SELECT_NET,
     PLACE_POWER_SYMBOL,
@@ -59,6 +60,7 @@ enum class ToolID {
     DRAW_LINE_RECTANGLE,
     EDIT_LINE_RECTANGLE,
     EDIT_SCHEMATIC_PROPERTIES,
+    EDIT_PROJECT_PROPERTIES,
     ROUTE_TRACK_INTERACTIVE,
     EDIT_VIA,
     ROTATE_ARBITRARY,
@@ -110,6 +112,7 @@ enum class ToolID {
     MOVE_KEY_FINE_RIGHT,
     SWAP_NETS,
     LINE_LOOP_TO_POLYGON,
+    LINES_TO_TRACKS,
     SCALE,
     CHANGE_UNIT,
     EDIT_FRAME_PROPERTIES,
@@ -159,5 +162,11 @@ enum class ToolID {
     MAP_PORT,
     ADD_BLOCK_INSTANCE,
     ALIGN_AND_DISTRIBUTE,
+    MERGE_DUPLICATE_JUNCTIONS,
+    EDIT_TEXT,
+    FLIP_NET_TIE,
+    TIE_NETS,
+    DRAW_NET_TIE,
+    MOVE_TRACK_CONNECTION,
 };
 } // namespace horizon
