@@ -623,10 +623,15 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
         }
         else if (args.type == ToolEventType::ACTION) {
             if (args.action == InToolActionID::LMB || (is_transient && args.action == InToolActionID::LMB_RELEASE)) {
+                if (doc.b) {
+                    auto &brd = *doc.b->get_board();
+                    brd.airwires_expand = nets;
+                    brd.expand_flags |= Board::EXPAND_AIRWIRES;
+                }
                 if (pic) {
                     return ToolResponse::commit();
                 }
-                merge_and_connect();
+                merge_and_connect({});
                 return ToolResponse::next(ToolResponse::Result::COMMIT, tool_id,
                                           std::make_unique<ToolDataPaste>(paste_data));
             }
