@@ -19,8 +19,9 @@
 #include "edit_shape.hpp"
 #include "pad_parameter_set_window.hpp"
 #include "schematic_properties.hpp"
+#include "project_properties.hpp"
 #include "edit_via.hpp"
-#include "edit_plane.hpp"
+#include "edit_plane_window.hpp"
 #include "edit_stackup.hpp"
 #include "edit_board_hole.hpp"
 #include "edit_frame.hpp"
@@ -37,12 +38,17 @@
 #include "enter_datum_scale_window.hpp"
 #include "router_settings_window.hpp"
 #include "edit_custom_value.hpp"
+#include "map_net_tie.hpp"
 #include <glibmm.h>
 #include "pool/ipool.hpp"
 #include "widgets/net_selector.hpp"
 #include "manage_ports.hpp"
 #include "select_block.hpp"
 #include "align_and_distribute_window.hpp"
+#include "edit_text_window.hpp"
+#include "plane_update.hpp"
+#include "util/automatic_prefs.hpp"
+#include "board/board.hpp"
 
 namespace horizon {
 void Dialogs::set_parent(Gtk::Window *w)
@@ -271,9 +277,16 @@ bool Dialogs::edit_board_hole(std::set<class BoardHole *> &holes, IPool &pool, I
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::edit_schematic_properties(class IDocumentSchematicBlockSymbol &doc)
+bool Dialogs::edit_schematic_properties(class IDocumentSchematicBlockSymbol &doc, const UUID &block, const UUID &sheet)
 {
     SchematicPropertiesDialog dia(parent, doc);
+    dia.select_sheet(block, sheet);
+    return dia.run() == Gtk::RESPONSE_OK;
+}
+
+bool Dialogs::edit_project_properties(class Block &block)
+{
+    ProjectPropertiesDialog dia(parent, block);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
@@ -310,19 +323,6 @@ std::optional<std::string> Dialogs::ask_datum_string(const std::string &label, c
     }
 }
 
-std::optional<std::string> Dialogs::ask_datum_string_multiline(const std::string &label, const std::string &def)
-{
-    AskDatumStringDialog dia(parent, label, TextEditor::Lines::MULTI);
-    dia.set_text(def);
-    auto r = dia.run();
-    if (r == Gtk::RESPONSE_OK) {
-        return dia.get_text();
-    }
-    else {
-        return {};
-    }
-}
-
 std::optional<Coordi> Dialogs::ask_datum_coord(const std::string &label, Coordi def)
 {
     AskDatumDialog dia(parent, label, true);
@@ -401,6 +401,18 @@ std::optional<UUID> Dialogs::map_package(const std::vector<std::pair<Component *
     }
 }
 
+std::optional<UUID> Dialogs::map_net_tie(const std::set<class NetTie *> &net_ties)
+{
+    MapNetTieDialog dia(parent, net_ties);
+    auto r = dia.run();
+    if (r == Gtk::RESPONSE_OK && dia.selection_valid) {
+        return dia.selected_uuid;
+    }
+    else {
+        return {};
+    }
+}
+
 std::optional<UUID> Dialogs::select_via_padstack(class IPool &pool)
 {
     PoolBrowserDialog dia(parent, ObjectType::PADSTACK, pool);
@@ -427,12 +439,6 @@ std::optional<UUID> Dialogs::select_decal(IPool &pool)
     }
 }
 
-bool Dialogs::edit_plane(class Plane &plane, class Board &brd)
-{
-    EditPlaneDialog dia(parent, plane, brd);
-    return dia.run() == Gtk::RESPONSE_OK;
-}
-
 bool Dialogs::edit_keepout(class Keepout &keepout, class IDocument &c, bool add_mode)
 {
     EditKeepoutDialog dia(parent, keepout, c, add_mode);
@@ -506,6 +512,80 @@ bool Dialogs::manage_ports(Block &b)
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
+static void bind_board_and_plane(SQLite::Query &q, const Board &brd, const Plane *plane)
+{
+    q.bind(1, brd.uuid);
+    if (plane)
+        q.bind(2, plane->uuid);
+    else
+        q.bind(2, UUID());
+}
+
+bool Dialogs::update_plane(class Board &brd, class Plane *plane)
+{
+    Glib::Timer timer;
+    bool retval;
+    bool dialog_was_shown = false;
+    {
+        PlaneUpdateDialog dia(*parent, brd, plane);
+        bool immediately_show_dialog = false;
+        SQLite::Query q(AutomaticPreferences::get().db,
+                        "SELECT visible FROM plane_update_dialog_visibility WHERE board = ? AND plane = ?");
+        bind_board_and_plane(q, brd, plane);
+        if (q.step()) {
+            immediately_show_dialog = q.get<int>(0);
+        }
+
+        if (!immediately_show_dialog) {
+            auto invisible = gtk_invisible_new();
+            gtk_grab_add(invisible);
+            parent->get_window()->set_cursor(Gdk::Cursor::create(parent->get_display(), "wait"));
+            auto loop = Glib::MainLoop::create();
+            auto timeout = Glib::signal_timeout().connect(
+                    [&loop] {
+                        if (loop->is_running())
+                            loop->quit();
+                        return false;
+                    },
+                    2000);
+            auto done = dia.signal_done().connect([&loop] {
+                if (loop->is_running())
+                    loop->quit();
+            });
+            loop->run();
+            gtk_grab_remove(invisible);
+            gtk_widget_destroy(invisible);
+
+            parent->get_window()->set_cursor();
+            done->disconnect();
+            timeout.disconnect();
+        }
+
+        if (!dia.is_done()) {
+            dialog_was_shown = true;
+            while (dia.run() != 1)
+                ;
+        }
+        retval = !dia.was_cancelled();
+    }
+    timer.stop();
+    bool show_dialog = dialog_was_shown;
+    if (timer.elapsed() < 2)
+        show_dialog = false;
+
+    if (retval) {
+        SQLite::Query q(
+                AutomaticPreferences::get().db,
+                "INSERT OR REPLACE INTO plane_update_dialog_visibility (board, plane, visible) VALUES (?, ?, ?)");
+        bind_board_and_plane(q, brd, plane);
+        q.bind(3, show_dialog);
+        q.step();
+    }
+
+    return retval;
+}
+
+
 class SymbolPinNamesWindow *Dialogs::show_symbol_pin_names_window(class SchematicSymbol &symbol)
 {
     if (window_nonmodal)
@@ -616,6 +696,33 @@ AlignAndDistributeWindow *Dialogs::show_align_and_distribute_window()
 }
 
 
+EditPlaneWindow *Dialogs::show_edit_plane_window(class Plane &plane, class Board &brd)
+{
+    if (auto win = dynamic_cast<EditPlaneWindow *>(window_nonmodal)) {
+        win->present();
+        return win;
+    }
+    auto win = new EditPlaneWindow(parent, interface, plane, brd);
+    window_nonmodal = win;
+    win->signal_hide().connect([this] { close_nonmodal(); });
+    win->present();
+    return win;
+}
+
+
+EditTextWindow *Dialogs::show_edit_text_window(class Text &text, bool use_ok)
+{
+    if (auto win = dynamic_cast<EditTextWindow *>(window_nonmodal)) {
+        win->present();
+        return win;
+    }
+    auto win = new EditTextWindow(parent, interface, text, use_ok);
+    window_nonmodal = win;
+    win->signal_hide().connect([this] { close_nonmodal(); });
+    win->present();
+    return win;
+}
+
 void Dialogs::close_nonmodal()
 {
     delete window_nonmodal;
