@@ -9,6 +9,7 @@
 #include "imp/action_catalog.hpp"
 #include "imp/actions.hpp"
 #include "preferences/preferences.hpp"
+#include "widgets/msd_tuning_window.hpp"
 
 namespace horizon {
 
@@ -75,7 +76,6 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
     settings_button->signal_toggled().connect(
             [settings_button, revealer] { revealer->set_reveal_child(settings_button->get_active()); });
 
-    Gtk::Button *update_button;
     GET_WIDGET(update_button);
     update_button->signal_clicked().connect([this] { update(); });
 
@@ -90,10 +90,7 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
             b->show();
             float az = it.azimuth;
             float el = it.elevation;
-            b->signal_clicked().connect([this, az, el] {
-                canvas->set_cam_azimuth(az);
-                canvas->set_cam_elevation(el);
-            });
+            b->signal_clicked().connect([this, az, el] { canvas->animate_to_azimuth_elevation_abs(az, el); });
             view_buttons_box->pack_start(*b, false, false, 0);
         }
     }
@@ -102,8 +99,8 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
     Gtk::Button *rotate_right_button;
     GET_WIDGET(rotate_left_button);
     GET_WIDGET(rotate_right_button);
-    rotate_left_button->signal_clicked().connect([this] { canvas->inc_cam_azimuth(-90); });
-    rotate_right_button->signal_clicked().connect([this] { canvas->inc_cam_azimuth(90); });
+    rotate_left_button->signal_clicked().connect([this] { canvas->animate_to_azimuth_elevation_rel(-90, 0); });
+    rotate_right_button->signal_clicked().connect([this] { canvas->animate_to_azimuth_elevation_rel(+90, 0); });
 
     Gtk::Button *view_all_button;
     GET_WIDGET(view_all_button);
@@ -255,15 +252,21 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
         }
     });
 
-    GET_WIDGET(model_loading_revealer);
-    GET_WIDGET(model_loading_spinner);
+    GET_WIDGET(loading_revealer);
+    GET_WIDGET(loading_spinner);
     GET_WIDGET(model_loading_progress);
+    GET_WIDGET(model_loading_box);
+    GET_WIDGET(layer_loading_progress);
+    GET_WIDGET(layer_loading_box);
     canvas->signal_models_loading().connect([this](unsigned int i, unsigned int n) {
-        bool loading = i < n;
-        model_loading_revealer->set_reveal_child(loading);
-        model_loading_spinner->property_active() = loading;
-        model_loading_progress->set_visible(n > 1);
-        model_loading_progress->set_fraction(i / (n * 1.0));
+        model_loading_i = i;
+        model_loading_n = n;
+        update_loading();
+    });
+    canvas->signal_layers_loading().connect([this](unsigned int i, unsigned int n) {
+        layer_loading_i = i;
+        layer_loading_n = n;
+        update_loading();
     });
 
     Gtk::ComboBoxText *msaa_combo;
@@ -293,6 +296,12 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
         }
     });
 
+    {
+        Gtk::Label *spacenav_label;
+        GET_WIDGET(spacenav_label);
+        spacenav_label->set_visible(canvas->get_spacenav_available());
+    }
+
     {
         Gtk::Box *lollipop_box;
         GET_WIDGET(lollipop_box);
@@ -318,6 +327,20 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
         }
     });
 
+    canvas->signal_spacenav_button_press().connect([this](unsigned int button) {
+        if (button < spnav_buttons.size()) {
+            auto act = spnav_buttons.at(button);
+            if (act != ActionID::NONE)
+                trigger_action(act);
+        }
+    });
+
+    msd_tuning_window = new MSDTuningWindow();
+    msd_tuning_window->set_transient_for(*this);
+    msd_tuning_window->signal_changed().connect(
+            [this] { canvas->set_msd_params(msd_tuning_window->get_msd_params()); });
+
+
     connect_action(ActionID::VIEW_ALL, [this](const auto &conn) { canvas->view_all(); });
     connect_action(ActionID::PAN_LEFT, sigc::mem_fun(*this, &View3DWindow::handle_pan_action));
     connect_action(ActionID::PAN_RIGHT, sigc::mem_fun(*this, &View3DWindow::handle_pan_action));
@@ -339,11 +362,38 @@ View3DWindow::View3DWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Buil
 
     connect_action(ActionID::VIEW_3D_ORTHO, sigc::mem_fun(*this, &View3DWindow::handle_proj_action));
     connect_action(ActionID::VIEW_3D_PERSP, sigc::mem_fun(*this, &View3DWindow::handle_proj_action));
+    connect_action(ActionID::VIEW_3D_TOGGLE_PERSP_ORTHO, sigc::mem_fun(*this, &View3DWindow::handle_proj_action));
 
+    connect_action(ActionID::MSD_TUNING_WINDOW, [this](const auto &conn) { msd_tuning_window->present(); });
 
     GET_WIDGET(main_box);
 }
 
+void View3DWindow::set_3d_title(const std::string &s)
+{
+    if (s.size())
+        set_title(s + " – 3D Preview");
+    else
+        set_title("3D Preview");
+}
+
+void View3DWindow::update_loading()
+{
+    const bool model_loading = model_loading_i < model_loading_n;
+    const bool layer_loading = layer_loading_i < layer_loading_n;
+    const bool loading = model_loading || layer_loading;
+    loading_revealer->set_reveal_child(loading);
+    loading_spinner->property_active() = loading;
+
+    if (loading)
+        model_loading_box->set_visible(model_loading);
+    model_loading_progress->set_visible(model_loading_n > 1);
+    model_loading_progress->set_fraction(model_loading_i / (model_loading_n * 1.0));
+
+    if (loading)
+        layer_loading_box->set_visible(layer_loading);
+    layer_loading_progress->set_fraction(layer_loading_i / (layer_loading_n * 1.0));
+}
 void View3DWindow::add_widget(Gtk::Widget *w)
 {
     main_box->pack_start(*w, false, false, 0);
@@ -351,11 +401,21 @@ void View3DWindow::add_widget(Gtk::Widget *w)
 
 void View3DWindow::update(bool clear)
 {
+    if (!needs_update)
+        return;
     s_signal_request_update.emit();
     canvas->update(board);
     if (clear)
         canvas->clear_3d_models();
     canvas->load_models_async(pool);
+    update_button->set_sensitive(false);
+    needs_update = false;
+}
+
+void View3DWindow::set_needs_update()
+{
+    needs_update = true;
+    update_button->set_sensitive(true);
 }
 
 void View3DWindow::set_highlights(const std::set<UUID> &pkgs)
@@ -535,7 +595,10 @@ ActionConnection &View3DWindow::connect_action(ActionID action_id, std::function
 void View3DWindow::apply_preferences(const Preferences &prefs)
 {
     canvas->smooth_zoom = prefs.zoom.smooth_zoom_3d;
+    canvas->touchpad_pan = prefs.zoom.touchpad_pan;
     canvas->set_appearance(prefs.canvas_layer.appearance);
+    canvas->spacenav_prefs = prefs.spacenav.prefs;
+    spnav_buttons = prefs.spacenav.buttons;
     const auto av = ActionCatalogItem::AVAILABLE_IN_3D;
     for (auto &it : action_connections) {
         const auto &act = action_catalog.at(it.first);
@@ -582,9 +645,8 @@ void View3DWindow::handle_pan_action(const ActionConnection &c)
         return;
     }
     d = d * 50;
-    auto center = canvas->get_center();
     auto shift = canvas->get_center_shift({d.x, d.y});
-    canvas->set_center(center + shift);
+    canvas->animate_center_rel(shift);
 }
 
 void View3DWindow::handle_zoom_action(const ActionConnection &conn)
@@ -593,7 +655,7 @@ void View3DWindow::handle_zoom_action(const ActionConnection &conn)
     if (conn.id.action == ActionID::ZOOM_IN)
         inc = -1;
 
-    canvas->set_cam_distance(canvas->get_cam_distance() * pow(1.5, inc));
+    canvas->animate_zoom_step(inc);
 }
 
 void View3DWindow::handle_rotate_action(const ActionConnection &conn)
@@ -601,15 +663,14 @@ void View3DWindow::handle_rotate_action(const ActionConnection &conn)
     auto inc = 90;
     if (conn.id.action == ActionID::ROTATE_VIEW_LEFT)
         inc = -90;
-    canvas->inc_cam_azimuth(inc);
+    canvas->animate_to_azimuth_elevation_rel(inc, 0);
 }
 
 void View3DWindow::handle_view_action(const ActionConnection &conn)
 {
     for (const auto &it : views) {
         if (it.action == conn.id.action) {
-            canvas->set_cam_azimuth(it.azimuth);
-            canvas->set_cam_elevation(it.elevation);
+            canvas->animate_to_azimuth_elevation_abs(it.azimuth, it.elevation);
             return;
         }
     }
@@ -617,10 +678,18 @@ void View3DWindow::handle_view_action(const ActionConnection &conn)
 
 void View3DWindow::handle_proj_action(const ActionConnection &conn)
 {
-    if (conn.id.action == ActionID::VIEW_3D_ORTHO)
+    if (conn.id.action == ActionID::VIEW_3D_ORTHO) {
         proj_ortho_rb->set_active(true);
-    else
+    }
+    else if (conn.id.action == ActionID::VIEW_3D_PERSP) {
         proj_persp_rb->set_active(true);
+    }
+    else {
+        if (proj_ortho_rb->get_active())
+            proj_persp_rb->set_active(true);
+        else
+            proj_ortho_rb->set_active(true);
+    }
 }
 
 
