@@ -256,6 +256,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
         {{ActionID::TOOL, ToolID::MAP_PIN},
          {"Place pin", ActionGroup::SYMBOL, ActionCatalogItem::AVAILABLE_IN_SYMBOL, ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::SELECT_CONNECTED_LINES},
+         {"Select connected lines", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::TOOL, ToolID::SELECT_NET_SEGMENT},
          {"Select net segment", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_DEFAULT}},
@@ -443,7 +447,11 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::EDIT_SCHEMATIC_PROPERTIES},
-         {"Edit schematic properties", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+         {"Manage sheets and blocks", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::EDIT_PROJECT_PROPERTIES},
+         {"Edit project properties", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
         {{ActionID::TOOL, ToolID::SET_DIFFPAIR},
@@ -625,6 +633,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Line loop to polygon", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::LINES_TO_TRACKS},
+         {"Lines to tracks", ActionGroup::GRAPHICS, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::RELOAD_POOL, ToolID::NONE},
          {"Reload pool", ActionGroup::UNKNOWN,
           ActionCatalogItem::AVAILABLE_IN_PACKAGE | ActionCatalogItem::AVAILABLE_IN_SYMBOL
@@ -943,6 +955,10 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Orthographic projection", ActionGroup::VIEW_3D, ActionCatalogItem::AVAILABLE_IN_3D,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::VIEW_3D_TOGGLE_PERSP_ORTHO, ToolID::NONE},
+         {"Toggle projection", ActionGroup::VIEW_3D, ActionCatalogItem::AVAILABLE_IN_3D,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::VIEW_3D_TOP, ToolID::NONE},
          {"View top", ActionGroup::VIEW_3D, ActionCatalogItem::AVAILABLE_IN_3D, ActionCatalogItem::FLAGS_DEFAULT}},
 
@@ -1009,10 +1025,49 @@ const std::map<ActionToolID, ActionCatalogItem> action_catalog = {
          {"Align and distribute", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_EVERYWHERE,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOOL, ToolID::MERGE_DUPLICATE_JUNCTIONS},
+         {"Merge duplicate junctions", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
         {{ActionID::GO_TO_PROJECT_MANAGER, ToolID::NONE},
          {"Go to project manager", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD,
           ActionCatalogItem::FLAGS_DEFAULT}},
 
+        {{ActionID::TOGGLE_JUNCTIONS_AND_HIDDEN_NAMES, ToolID::NONE},
+         {"Toggle junctions and hidden names", ActionGroup::SYMBOL, ActionCatalogItem::AVAILABLE_IN_SYMBOL,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::EDIT_TEXT},
+         {"Edit text", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::MSD_TUNING_WINDOW, ToolID::NONE},
+         {"Tune MSD", ActionGroup::UNKNOWN, ActionCatalogItem::AVAILABLE_EVERYWHERE_3D,
+          ActionCatalogItem::FLAGS_NO_POPOVER}},
+
+        {{ActionID::TOOL, ToolID::TIE_NETS},
+         {"Tie nets", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::DRAW_NET_TIE},
+         {"Draw net tie", ActionGroup::BOARD, ActionCatalogItem::AVAILABLE_IN_BOARD, ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::FLIP_NET_TIE},
+         {"Flip net tie", ActionGroup::SCHEMATIC, ActionCatalogItem::AVAILABLE_IN_SCHEMATIC,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOOL, ToolID::MOVE_TRACK_CONNECTION},
+         {"Move track connection", ActionGroup::MOVE, ActionCatalogItem::AVAILABLE_IN_BOARD,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::TOGGLE_PICTURES, ToolID::NONE},
+         {"Toggle pictures", ActionGroup::VIEW,
+          ActionCatalogItem::AVAILABLE_IN_SCHEMATIC_AND_BOARD | ActionCatalogItem::AVAILABLE_IN_PACKAGE,
+          ActionCatalogItem::FLAGS_DEFAULT}},
+
+        {{ActionID::VIEW_ACTUAL_SIZE, ToolID::NONE},
+         {"View at actual size", ActionGroup::VIEW, ActionCatalogItem::AVAILABLE_EVERYWHERE,
+          ActionCatalogItem::FLAGS_IN_TOOL}},
 };
 
 const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
@@ -1045,6 +1100,7 @@ const std::vector<std::pair<ActionGroup, std::string>> action_group_catalog = {
     }
 
 const LutEnumStr<ActionID> action_lut = {
+        ACTION_LUT_ITEM(NONE),
         ACTION_LUT_ITEM(TOOL),
         ACTION_LUT_ITEM(SELECTION_FILTER),
         ACTION_LUT_ITEM(DISTRACTION_FREE),
@@ -1144,6 +1200,7 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(PREV_SHEET),
         ACTION_LUT_ITEM(VIEW_3D_PERSP),
         ACTION_LUT_ITEM(VIEW_3D_ORTHO),
+        ACTION_LUT_ITEM(VIEW_3D_TOGGLE_PERSP_ORTHO),
         ACTION_LUT_ITEM(VIEW_3D_FRONT),
         ACTION_LUT_ITEM(VIEW_3D_BACK),
         ACTION_LUT_ITEM(VIEW_3D_TOP),
@@ -1157,6 +1214,10 @@ const LutEnumStr<ActionID> action_lut = {
         ACTION_LUT_ITEM(EDIT_BLOCK_SYMBOL),
         ACTION_LUT_ITEM(GO_TO_BLOCK_SYMBOL),
         ACTION_LUT_ITEM(GO_TO_PROJECT_MANAGER),
+        ACTION_LUT_ITEM(TOGGLE_JUNCTIONS_AND_HIDDEN_NAMES),
+        ACTION_LUT_ITEM(MSD_TUNING_WINDOW),
+        ACTION_LUT_ITEM(TOGGLE_PICTURES),
+        ACTION_LUT_ITEM(VIEW_ACTUAL_SIZE),
 };
 
 #define TOOL_LUT_ITEM(x)                                                                                               \
@@ -1166,6 +1227,7 @@ const LutEnumStr<ActionID> action_lut = {
 
 const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(EDIT_SCHEMATIC_PROPERTIES),
+        TOOL_LUT_ITEM(EDIT_PROJECT_PROPERTIES),
         TOOL_LUT_ITEM(NONE),
         TOOL_LUT_ITEM(MOVE),
         TOOL_LUT_ITEM(PLACE_JUNCTION),
@@ -1269,6 +1331,7 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(MOVE_KEY_FINE_RIGHT),
         TOOL_LUT_ITEM(SWAP_NETS),
         TOOL_LUT_ITEM(LINE_LOOP_TO_POLYGON),
+        TOOL_LUT_ITEM(LINES_TO_TRACKS),
         TOOL_LUT_ITEM(SCALE),
         TOOL_LUT_ITEM(CHANGE_UNIT),
         TOOL_LUT_ITEM(EDIT_FRAME_PROPERTIES),
@@ -1318,5 +1381,11 @@ const LutEnumStr<ToolID> tool_lut = {
         TOOL_LUT_ITEM(MAP_PORT),
         TOOL_LUT_ITEM(ADD_BLOCK_INSTANCE),
         TOOL_LUT_ITEM(ALIGN_AND_DISTRIBUTE),
+        TOOL_LUT_ITEM(MERGE_DUPLICATE_JUNCTIONS),
+        TOOL_LUT_ITEM(EDIT_TEXT),
+        TOOL_LUT_ITEM(TIE_NETS),
+        TOOL_LUT_ITEM(DRAW_NET_TIE),
+        TOOL_LUT_ITEM(FLIP_NET_TIE),
+        TOOL_LUT_ITEM(MOVE_TRACK_CONNECTION),
 };
 } // namespace horizon
