@@ -103,6 +103,7 @@ enum class ActionID {
     PREV_SHEET,
     VIEW_3D_PERSP,
     VIEW_3D_ORTHO,
+    VIEW_3D_TOGGLE_PERSP_ORTHO,
     VIEW_3D_FRONT,
     VIEW_3D_BACK,
     VIEW_3D_TOP,
@@ -116,5 +117,9 @@ enum class ActionID {
     EDIT_BLOCK_SYMBOL,
     GO_TO_BLOCK_SYMBOL,
     GO_TO_PROJECT_MANAGER,
+    TOGGLE_JUNCTIONS_AND_HIDDEN_NAMES,
+    MSD_TUNING_WINDOW,
+    TOGGLE_PICTURES,
+    VIEW_ACTUAL_SIZE,
 };
 }
