@@ -35,6 +35,7 @@
 #include "pool/pool_parametric.hpp"
 #include "action_icon.hpp"
 #include "grids_window.hpp"
+#include "widgets/msd_tuning_window.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -156,6 +157,7 @@ json ImpBase::send_json(const json &j)
         return nullptr;
 
     std::string s = j.dump();
+    const std::string op = j.value("op", "unknown");
     zmq::message_t msg(UUID::size + s.size() + 1);
     auto m = (char *)msg.data();
     memcpy(m, ipc_cookie.get_bytes(), UUID::size);
@@ -165,14 +167,14 @@ json ImpBase::send_json(const json &j)
         if (zmq_helper::send(sock_project, msg) == false) {
             sockets_broken = true;
             sockets_connected = false;
-            show_sockets_broken_dialog("send timeout");
+            show_sockets_broken_dialog("send timeout, op=" + op);
             return nullptr;
         }
     }
     catch (zmq::error_t &e) {
         sockets_broken = true;
         sockets_connected = false;
-        show_sockets_broken_dialog(e.what());
+        show_sockets_broken_dialog(e.what() + std::string(", op=") + op);
         return nullptr;
     }
 
@@ -181,14 +183,14 @@ json ImpBase::send_json(const json &j)
         if (zmq_helper::recv(sock_project, rx) == false) {
             sockets_broken = true;
             sockets_connected = false;
-            show_sockets_broken_dialog("receive timeout");
+            show_sockets_broken_dialog("receive timeout, op=" + op);
             return nullptr;
         }
     }
     catch (zmq::error_t &e) {
         sockets_broken = true;
         sockets_connected = false;
-        show_sockets_broken_dialog(e.what());
+        show_sockets_broken_dialog(e.what() + std::string(", op=") + op);
         return nullptr;
     }
     char *rxdata = ((char *)rx.data());
@@ -222,10 +224,15 @@ bool ImpBase::handle_close(const GdkEventAny *ev)
             return false; // close
 
         case 2:
-            force_end_tool();
-            trigger_action(ActionID::SAVE);
-            core->delete_autosave();
-            return false; // close
+            if (force_end_tool()) {
+                trigger_action(ActionID::SAVE);
+                core->delete_autosave();
+                return false; // close
+            }
+            else {
+                return true;
+            }
+
 
         default:
             return true; // keep window open
@@ -500,6 +507,23 @@ void ImpBase::run(int argc, char *argv[])
         main_window->property_scrolled_window->set_visible(show_properties && !distraction_free);
         this->update_view_hints();
     });
+
+    if (core->has_object_type(ObjectType::PICTURE)) {
+        show_pictures_action = main_window->add_action_bool("show_pictures", canvas->show_pictures);
+        show_pictures_action->signal_change_state().connect([this](const Glib::VariantBase &v) {
+            auto b = Glib::VariantBase::cast_dynamic<Glib::Variant<bool>>(v).get();
+            if (b != canvas->show_pictures) {
+                trigger_action(ActionID::TOGGLE_PICTURES);
+            }
+        });
+        connect_action(ActionID::TOGGLE_PICTURES, [this](const auto &a) {
+            canvas->show_pictures = !canvas->show_pictures;
+            canvas->queue_draw();
+            g_simple_action_set_state(show_pictures_action->gobj(), g_variant_new_boolean(canvas->show_pictures));
+            update_view_hints();
+        });
+    }
+
     connect_action(ActionID::SELECTION_FILTER, [this](const auto &a) { selection_filter_dialog->present(); });
     connect_action(ActionID::SAVE, [this](const auto &a) {
         if (!read_only) {
@@ -711,7 +735,7 @@ void ImpBase::run(int argc, char *argv[])
             [this](ToolID id, json j) { save_json_to_file(get_tool_settings_filename(id), j); });
 
     if (core->get_rules()) {
-        rules_window = RulesWindow::create(main_window, *canvas, *core);
+        rules_window = RulesWindow::create(main_window, *canvas, *core, is_layered());
         rules_window->signal_canvas_update().connect(sigc::mem_fun(*this, &ImpBase::canvas_update_from_pp));
         rules_window->signal_changed().connect([this] { core->set_needs_save(); });
         core->signal_tool_changed().connect([this](ToolID id) { rules_window->set_enabled(id == ToolID::NONE); });
@@ -731,6 +755,12 @@ void ImpBase::run(int argc, char *argv[])
         }
     }
 
+    msd_tuning_window = new MSDTuningWindow();
+    msd_tuning_window->set_transient_for(*main_window);
+    msd_tuning_window->signal_changed().connect(
+            [this] { canvas->set_msd_params(msd_tuning_window->get_msd_params()); });
+    connect_action(ActionID::MSD_TUNING_WINDOW, [this](const auto &conn) { msd_tuning_window->present(); });
+
     tool_popover = Gtk::manage(new ToolPopover(canvas, get_editor_type_for_action()));
     tool_popover->set_position(Gtk::POS_BOTTOM);
     tool_popover->signal_action_activated().connect([this](ActionToolID action_id) { trigger_action(action_id); });
@@ -738,8 +768,8 @@ void ImpBase::run(int argc, char *argv[])
 
     log_window = new LogWindow();
     log_window->set_transient_for(*main_window);
-    Logger::get().set_log_handler([this](const Logger::Item &it) { log_window->get_view()->push_log(it); });
-
+    log_dispatcher.set_handler([this](const auto &it) { log_window->get_view()->push_log(it); });
+    Logger::get().set_log_handler([this](const Logger::Item &it) { log_dispatcher.log(it); });
     main_window->add_action("view_log", [this] { log_window->present(); });
 
     main_window->signal_delete_event().connect(sigc::mem_fun(*this, &ImpBase::handle_close));
@@ -780,6 +810,8 @@ void ImpBase::run(int argc, char *argv[])
     view_options_menu_append_action("Distraction free mode", "win.distraction_free");
     add_tool_action(ActionID::SELECTION_FILTER, "selection_filter");
     view_options_menu_append_action("Selection filter", "win.selection_filter");
+    if (core->has_object_type(ObjectType::PICTURE))
+        view_options_menu_append_action("Pictures", "win.show_pictures");
 
     imp_interface = std::make_unique<ImpInterface>(this);
 
@@ -1564,8 +1596,8 @@ bool ImpBase::handle_broadcast(const json &j)
         return true;
     }
     else if (op == "save") {
-        force_end_tool();
-        trigger_action(ActionID::SAVE);
+        if (force_end_tool())
+            trigger_action(ActionID::SAVE);
         return true;
     }
     else if (op == "close") {
@@ -1639,7 +1671,7 @@ ActionToolID ImpBase::get_doubleclick_action(ObjectType type, const UUID &uu)
 {
     switch (type) {
     case ObjectType::TEXT:
-        return ToolID::ENTER_DATUM;
+        return ToolID::EDIT_TEXT;
 
     case ObjectType::POLYGON_ARC_CENTER:
     case ObjectType::POLYGON_VERTEX:
@@ -1682,7 +1714,7 @@ void ImpBase::load_meta()
 void ImpBase::set_window_title(const std::string &s)
 {
     if (s.size()) {
-        main_window->set_title(s + " - " + object_descriptions.at(get_editor_type()).name);
+        main_window->set_title(s + " – " + object_descriptions.at(get_editor_type()).name);
     }
     else {
         main_window->set_title("Untitled " + object_descriptions.at(get_editor_type()).name);
@@ -1707,6 +1739,9 @@ std::vector<std::string> ImpBase::get_view_hints()
 
     if (selection_filter_dialog->get_filtered())
         r.emplace_back("selection filtered");
+
+    if (!canvas->show_pictures)
+        r.emplace_back("no pictures");
     return r;
 }
 
