@@ -21,6 +21,7 @@
 #include "core/clipboard/clipboard.hpp"
 #include "clipboard_handler.hpp"
 #include "util/win32_undef.hpp"
+#include "logger/log_dispatcher.hpp"
 
 namespace horizon {
 
@@ -204,7 +205,7 @@ protected:
     {
     }
 
-    void set_window_title(const std::string &s);
+    virtual void set_window_title(const std::string &s);
     void set_window_title_from_block();
 
     void update_view_hints();
@@ -237,7 +238,7 @@ protected:
     class ActionButton &add_action_button_line();
 
     virtual ToolID get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const;
-    void force_end_tool();
+    bool force_end_tool();
     void reset_tool_hint_label();
 
     std::set<ObjectRef> highlights;
@@ -293,6 +294,7 @@ private:
     void set_search_mode(bool enabled, bool focus = true);
     std::map<Searcher::Type, Gtk::CheckButton *> search_check_buttons;
 
+    LogDispatcher log_dispatcher;
     class LogWindow *log_window = nullptr;
     std::set<SelectableRef> selection_for_drag_move;
     ToolID drag_tool;
@@ -317,6 +319,7 @@ private:
 
     Glib::RefPtr<Gio::SimpleAction> bottom_view_action;
     Glib::RefPtr<Gio::SimpleAction> distraction_free_action;
+    Glib::RefPtr<Gio::SimpleAction> show_pictures_action;
 
     int left_panel_width = 0;
 
@@ -349,5 +352,7 @@ private:
     Gtk::Button *redo_button = nullptr;
 
     unsigned int saved_version = 0;
+
+    class MSDTuningWindow *msd_tuning_window = nullptr;
 };
 } // namespace horizon
