@@ -167,23 +167,24 @@ bool ImpBoard::handle_broadcast(const json &j)
             update_highlights();
         }
         else if (op == "place") {
-            force_end_tool();
             main_window->present(timestamp);
-            std::set<SelectableRef> components;
-            const json &o = j["components"];
-            for (auto it = o.cbegin(); it != o.cend(); ++it) {
-                auto type = static_cast<ObjectType>(it.value().at("type").get<int>());
-                if (type == ObjectType::COMPONENT) {
-                    UUID uu(it.value().at("uuid").get<std::string>());
-                    components.emplace(uu, type);
+            if (force_end_tool()) {
+                std::set<SelectableRef> components;
+                const json &o = j["components"];
+                for (auto it = o.cbegin(); it != o.cend(); ++it) {
+                    auto type = static_cast<ObjectType>(it.value().at("type").get<int>());
+                    if (type == ObjectType::COMPONENT) {
+                        UUID uu(it.value().at("uuid").get<std::string>());
+                        components.emplace(uu, type);
+                    }
                 }
+                tool_begin(ToolID::MAP_PACKAGE, true, components);
             }
-            tool_begin(ToolID::MAP_PACKAGE, true, components);
         }
         else if (op == "reload-netlist") {
-            force_end_tool();
             main_window->present(timestamp);
-            trigger_action(ActionID::RELOAD_NETLIST);
+            if (force_end_tool())
+                trigger_action(ActionID::RELOAD_NETLIST);
         }
         else if (op == "reload-netlist-hint" && !core->tool_is_active()) {
             reload_netlist_delay_conn = Glib::signal_timeout().connect(
@@ -613,6 +614,7 @@ void ImpBoard::construct()
                 canvas->set_selection({});
         }
     });
+    core_board.signal_rebuilt().connect([this] { view_3d_window->set_needs_update(); });
 
     step_export_window = StepExportWindow::create(main_window, core_board, project_dir);
     step_export_window->signal_changed().connect([this] { core_board.set_needs_save(); });
@@ -855,6 +857,12 @@ void ImpBoard::construct()
     set_view_angle(0);
 }
 
+void ImpBoard::set_window_title(const std::string &s)
+{
+    view_3d_window->set_3d_title(s);
+    ImpBase::set_window_title(s);
+}
+
 UUID ImpBoard::net_from_selectable(const SelectableRef &sr)
 {
     const auto &board = *core_board.get_board();
@@ -929,7 +937,7 @@ void ImpBoard::update_text_owner_annotation()
 std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
 {
     std::string s;
-    if (sel_count_type(sel, ObjectType::TRACK)) {
+    if (sel_has_type(sel, ObjectType::TRACK)) {
         auto n = sel_count_type(sel, ObjectType::TRACK);
         s += "\n\n<b>" + std::to_string(n) + " " + object_descriptions.at(ObjectType::TRACK).get_name_for_n(n)
              + "</b>\n";
@@ -980,7 +988,7 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
     trim(s);
     if (auto it_sel = sel_find_exactly_one(sel, ObjectType::BOARD_PACKAGE)) {
         const auto &pkg = core_board.get_board()->packages.at(it_sel->uuid);
-        s += "\n\n<b>Package " + pkg.component->refdes + "</b>";
+        s += "\n\n<b>Package " + Glib::Markup::escape_text(pkg.component->refdes) + "</b>";
         if (pkg.fixed) {
             s += " (not movable)";
         }
@@ -1010,13 +1018,13 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
     if (sel_count_type(sel, ObjectType::POLYGON_VERTEX) == 1 || sel_count_type(sel, ObjectType::POLYGON_EDGE) == 1) {
         const auto &brd = *core_board.get_board();
         const Polygon *poly = nullptr;
-        if (sel_count_type(sel, ObjectType::POLYGON_VERTEX))
+        if (sel_has_type(sel, ObjectType::POLYGON_VERTEX))
             poly = &brd.polygons.at(sel_find_one(sel, ObjectType::POLYGON_VERTEX).uuid);
-        if (sel_count_type(sel, ObjectType::POLYGON_EDGE))
+        if (sel_has_type(sel, ObjectType::POLYGON_EDGE))
             poly = &brd.polygons.at(sel_find_one(sel, ObjectType::POLYGON_EDGE).uuid);
         if (poly) {
             if (auto plane = dynamic_cast<const Plane *>(poly->usage.ptr)) {
-                s += "\n\n<b>Plane " + plane->net->name + "</b>\n";
+                s += "\n\n<b>Plane " + Glib::Markup::escape_text(plane->net->name) + "</b>\n";
                 s += "Fill order: " + std::to_string(plane->priority) + "\n";
                 s += "Layer: ";
                 s += core_board.get_layer_provider().get_layers().at(poly->layer).name + "\n";
@@ -1254,6 +1262,7 @@ std::map<ObjectType, ImpBase::SelectionFilterInfo> ImpBoard::get_selection_filte
     std::map<ObjectType, ImpBase::SelectionFilterInfo> r = {
             {ObjectType::BOARD_PACKAGE, {layers_package, Flag::DEFAULT}},
             {ObjectType::TRACK, {layers_track, Flag::DEFAULT}},
+            {ObjectType::BOARD_NET_TIE, {layers_track, Flag::DEFAULT}},
             {ObjectType::VIA, {{}, Flag::WORK_LAYER_ONLY_ENABLED}},
             {ObjectType::BOARD_DECAL, {}},
             {ObjectType::POLYGON, {layers_polygon, Flag::HAS_OTHERS}},
