@@ -18,6 +18,7 @@
 #include "core/tool_id.hpp"
 #include "actions.hpp"
 #include "widgets/action_button.hpp"
+#include <iostream>
 
 namespace horizon {
 ImpSchematic::ImpSchematic(const std::string &blocks_filename, const std::string &pictures_dir,
@@ -373,15 +374,18 @@ bool ImpSchematic::handle_broadcast(const json &j)
         const auto op = j.at("op").get<std::string>();
         guint32 timestamp = j.value("time", 0);
         if (op == "place-part") {
-            force_end_tool();
             main_window->present(timestamp);
-            auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
-            tool_begin(ToolID::ADD_PART, false, {}, std::move(data));
+            if (force_end_tool()) {
+                auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
+                tool_begin(ToolID::ADD_PART, false, {}, std::move(data));
+            }
         }
         else if (op == "assign-part" && !core->tool_is_active()) {
             main_window->present(timestamp);
-            auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
-            tool_begin(ToolID::ASSIGN_PART, false, {}, std::move(data));
+            if (force_end_tool()) {
+                auto data = std::make_unique<ToolAddPart::ToolDataAddPart>(j.at("part").get<std::string>());
+                tool_begin(ToolID::ASSIGN_PART, false, {}, std::move(data));
+            }
         }
         else if (op == "highlight" && cross_probing_enabled && !core->tool_is_active()) {
             clear_highlights();
@@ -417,15 +421,15 @@ bool ImpSchematic::handle_broadcast(const json &j)
                     data->connections.push_back(item);
             }
             if (data->connections.size()) {
-                force_end_tool();
                 main_window->present(timestamp);
-                tool_begin(ToolID::BACKANNOTATE_CONNECTION_LINES, true, {}, std::move(data));
+                if (force_end_tool())
+                    tool_begin(ToolID::BACKANNOTATE_CONNECTION_LINES, true, {}, std::move(data));
             }
         }
         else if (op == "edit-meta") {
-            force_end_tool();
             main_window->present(timestamp);
-            tool_begin(ToolID::EDIT_SCHEMATIC_PROPERTIES);
+            if (force_end_tool())
+                tool_begin(ToolID::EDIT_PROJECT_PROPERTIES);
         }
     }
     return true;
@@ -517,6 +521,20 @@ void ImpSchematic::handle_show_in_pool_manager(const ActionConnection &conn)
     }
 }
 
+static void block_to_board(json &j, const Block &block, UUIDVec instance_path)
+{
+    if (Block::instance_path_too_long(instance_path, __FUNCTION__))
+        return;
+    for (const auto &[uu, comp] : block.components) {
+        json k;
+        k["type"] = static_cast<int>(ObjectType::COMPONENT);
+        k["uuid"] = (std::string)uuid_vec_flatten(uuid_vec_append(instance_path, uu));
+        j.push_back(k);
+    }
+    for (const auto &[uu, inst] : block.block_instances) {
+        block_to_board(j, *inst.block, uuid_vec_append(instance_path, uu));
+    }
+}
 
 void ImpSchematic::construct()
 {
@@ -553,9 +571,12 @@ void ImpSchematic::construct()
     hamburger_menu->append("Power Nets", "win.manage_pn");
     add_tool_action(ToolID::MANAGE_POWER_NETS, "manage_pn");
 
-    hamburger_menu->append("Schematic properties", "win.sch_properties");
+    hamburger_menu->append("Sheets and blocks", "win.sch_properties");
     add_tool_action(ToolID::EDIT_SCHEMATIC_PROPERTIES, "sch_properties");
 
+    hamburger_menu->append("Project properties", "win.prj_properties");
+    add_tool_action(ToolID::EDIT_PROJECT_PROPERTIES, "prj_properties");
+
     hamburger_menu->append("Export BOM…", "win.bom_export");
     main_window->add_action("bom_export", [this] { trigger_action(ActionID::BOM_EXPORT_WINDOW); });
 
@@ -610,6 +631,11 @@ void ImpSchematic::construct()
                             uuid_vec_append(core_schematic.get_instance_path(), sym.component->uuid));
                     j["selection"].push_back(k);
                 }
+                else if (it.type == ObjectType::SCHEMATIC_BLOCK_SYMBOL) {
+                    const auto &sym = sheet->block_symbols.at(it.uuid);
+                    block_to_board(j["selection"], *sym.block_instance->block,
+                                   uuid_vec_append(core_schematic.get_instance_path(), sym.block_instance->uuid));
+                }
             }
             canvas->set_selection({});
             allow_set_foreground_window(this->get_board_pid());
@@ -867,6 +893,7 @@ ActionButton &ImpSchematic::add_action_button_schematic(ActionToolID id)
 void ImpSchematic::update_action_sensitivity()
 {
     auto sel = canvas->get_selection();
+    const auto n_block_sym = sel_count_type(sel, ObjectType::SCHEMATIC_BLOCK_SYMBOL);
     if (sockets_connected) {
         json req;
         req["op"] = "has-board";
@@ -881,7 +908,7 @@ void ImpSchematic::update_action_sensitivity()
             set_action_sensitive(ActionID::TO_BOARD, false);
         }
         else {
-            set_action_sensitive(ActionID::TO_BOARD, n_sym && core_schematic.in_hierarchy());
+            set_action_sensitive(ActionID::TO_BOARD, (n_sym || n_block_sym) && core_schematic.in_hierarchy());
         }
     }
     else {
@@ -924,7 +951,8 @@ void ImpSchematic::update_action_sensitivity()
 
     set_action_sensitive(ActionID::HIGHLIGHT_GROUP, can_higlight_group);
     set_action_sensitive(ActionID::HIGHLIGHT_TAG, can_higlight_tag);
-    bool have_block_sym = sel_count_type(sel, ObjectType::SCHEMATIC_BLOCK_SYMBOL) == 1;
+
+    const auto have_block_sym = n_block_sym == 1;
     set_action_sensitive(ActionID::PUSH_INTO_BLOCK, have_block_sym && core_schematic.in_hierarchy());
     set_action_sensitive(ActionID::EDIT_BLOCK_SYMBOL, have_block_sym);
     set_action_sensitive(ActionID::POP_OUT_OF_BLOCK, core_schematic.get_instance_path().size());
@@ -955,6 +983,7 @@ std::map<ObjectType, ImpBase::SelectionFilterInfo> ImpSchematic::get_selection_f
         case ObjectType::POWER_SYMBOL:
         case ObjectType::BLOCK_SYMBOL_PORT:
         case ObjectType::SCHEMATIC_BLOCK_SYMBOL:
+        case ObjectType::SCHEMATIC_NET_TIE:
             r[type];
             break;
         default:;
@@ -1234,6 +1263,11 @@ void ImpSchematic::handle_move_to_other_sheet(const ActionConnection &conn)
                 auto &rip = core_schematic.get_sheet()->bus_rippers.at(it.uuid);
                 new_sel.emplace(rip.junction->uuid, ObjectType::JUNCTION);
             } break;
+            case ObjectType::SCHEMATIC_NET_TIE: {
+                auto &tie = core_schematic.get_sheet()->net_ties.at(it.uuid);
+                new_sel.emplace(tie.from->uuid, ObjectType::JUNCTION);
+                new_sel.emplace(tie.to->uuid, ObjectType::JUNCTION);
+            } break;
 
             case ObjectType::LINE_NET: {
                 auto line = &core_schematic.get_sheet()->net_lines.at(it.uuid);
@@ -1342,6 +1376,13 @@ void ImpSchematic::handle_move_to_other_sheet(const ActionConnection &conn)
                 }
             }
         }
+        for (const auto &it : core_schematic.get_sheet()->net_ties) {
+            for (const auto &it_ft : {it.second.from, it.second.to}) {
+                if (selection.count(SelectableRef(it_ft->uuid, ObjectType::JUNCTION))) {
+                    new_sel.emplace(it.first, ObjectType::SCHEMATIC_NET_TIE);
+                }
+            }
+        }
 
         added = false;
         for (const auto &it : new_sel) {
@@ -1409,6 +1450,10 @@ void ImpSchematic::handle_move_to_other_sheet(const ActionConnection &conn)
             new_sheet->lines.insert(std::make_pair(it.uuid, std::move(old_sheet->lines.at(it.uuid))));
             old_sheet->lines.erase(it.uuid);
         } break;
+        case ObjectType::SCHEMATIC_NET_TIE: {
+            new_sheet->net_ties.insert(std::make_pair(it.uuid, std::move(old_sheet->net_ties.at(it.uuid))));
+            old_sheet->net_ties.erase(it.uuid);
+        } break;
         case ObjectType::ARC: {
             new_sheet->arcs.insert(std::make_pair(it.uuid, std::move(old_sheet->arcs.at(it.uuid))));
             old_sheet->arcs.erase(it.uuid);
@@ -1547,7 +1592,7 @@ void ImpSchematic::update_monitor()
     ItemSet mon_items;
     for (const auto &[uu, block] : core_schematic.get_blocks().blocks) {
         {
-            ItemSet items = block.schematic.get_pool_items_used();
+            ItemSet items = block.block.get_pool_items_used();
             mon_items.insert(items.begin(), items.end());
         }
         {
