@@ -531,15 +531,19 @@ void PartWizard::handle_import()
     GtkFileChooserNative *native =
             gtk_file_chooser_native_new("Open", gobj(), GTK_FILE_CHOOSER_ACTION_OPEN, "_Open", "_Cancel");
     auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
-    auto filter = Gtk::FileFilter::create();
-    filter->set_name("CSV documents");
-    filter->add_pattern("*.csv");
-    chooser->add_filter(filter);
-    filter = Gtk::FileFilter::create();
-    filter->set_name("json documents");
-    filter->add_pattern("*.json");
-    chooser->add_filter(filter);
-
+    {
+        auto filter = Gtk::FileFilter::create();
+        filter->set_name("Supported files (csv and json)");
+        filter->add_pattern("*.csv");
+        filter->add_pattern("*.json");
+        chooser->add_filter(filter);
+    }
+    {
+        auto filter = Gtk::FileFilter::create();
+        filter->set_name("All files");
+        filter->add_pattern("*");
+        chooser->add_filter(filter);
+    }
     if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
         auto filename = chooser->get_filename();
         try {
@@ -607,7 +611,7 @@ void PartWizard::import_pads(CSV::Csv &csv)
                 std::string alt = line[i];
                 trim(alt);
                 if (alt.size()) {
-                    item.alt.push_back(alt);
+                    item.alt.push_back({alt, item.direction});
                 }
             }
         }
@@ -627,7 +631,13 @@ void PartWizard::import_pads(const json &j)
         item.direction = Pin::direction_lut.lookup(v.value("direction", ""), Pin::Direction::INPUT);
         if (v.count("alt")) {
             for (const auto &a : v.at("alt")) {
-                item.alt.push_back(a.get<std::string>());
+                if (a.is_string()) {
+                    item.alt.push_back({a.get<std::string>(), item.direction});
+                }
+                else if (a.is_object()) {
+                    item.alt.push_back({a.at("name").get<std::string>(),
+                                        Pin::direction_lut.lookup(a.value("direction", ""), item.direction)});
+                }
             }
         }
     }
@@ -654,13 +664,12 @@ void PartWizard::import_pads(const std::map<std::string, PadImportItem> &items)
             ed->pin_name_entry->set_text(pin_name);
             ed->dir_combo->set_active_id(std::to_string(static_cast<int>(it.direction)));
             ed->combo_gate_entry->set_text(gate_name);
-            {
-                std::stringstream ss;
-                for (auto &it2 : it.alt) {
-                    ss << it2 << " ";
-                }
-                ed->pin_names_entry->set_text(ss.str());
+            ed->pin_names.clear();
+            for (const auto &alt : it.alt) {
+                ed->pin_names.emplace(std::piecewise_construct, std::forward_as_tuple(UUID::random()),
+                                      std::forward_as_tuple(alt.name, alt.direction));
             }
+            ed->pin_names_editor->reload();
         }
     }
     autolink_pads();
@@ -715,7 +724,6 @@ void PartWizard::prepare_edit()
                 delete ed;
             }
             else {
-                std::cout << "found gate " << (std::string)ed->gate->uuid << std::endl;
                 gates_avail.insert(ed->gate);
             }
         }
@@ -737,18 +745,21 @@ void PartWizard::prepare_edit()
                 }
 
                 auto proc = appwin.spawn(PoolProjectManagerProcess::Type::IMP_SYMBOL, {symbol_filename});
-                processes.emplace(symbol_filename, proc);
-                symbols_open.emplace(symbol_uuid);
-                proc->signal_exited().connect([this, symbol_filename, symbol_uuid, ed](int status, bool modified) {
-                    processes.erase(symbol_filename);
-                    symbols_open.erase(symbol_uuid);
-                    {
-                        auto sym = Symbol::new_from_file(symbol_filename, pool);
-                        ed->symbol_name_entry->set_text(sym.name);
-                    }
-                    update_can_finish();
-                    update_symbol_pins_mapped();
-                });
+                if (proc.spawned) {
+                    processes.emplace(symbol_filename, proc.proc);
+                    symbols_open.emplace(symbol_uuid);
+                    proc.proc->signal_exited().connect(
+                            [this, symbol_filename, symbol_uuid, ed](int status, bool modified) {
+                                processes.erase(symbol_filename);
+                                symbols_open.erase(symbol_uuid);
+                                {
+                                    auto sym = Symbol::new_from_file(symbol_filename, pool);
+                                    ed->symbol_name_entry->set_text(sym.name);
+                                }
+                                update_can_finish();
+                                update_symbol_pins_mapped();
+                            });
+                }
                 update_can_finish();
             });
 
@@ -794,7 +805,6 @@ void PartWizard::update_pin_warnings()
 
 void PartWizard::update_part()
 {
-    std::cout << "upd part" << std::endl;
     std::set<UUID> pins_used;
     std::set<UUID> units_used;
     part.pad_map.clear();
@@ -824,13 +834,8 @@ void PartWizard::update_part()
             pins_used.insert(pin->uuid);
             pin->primary_name = pin_name;
             pin->direction = static_cast<Pin::Direction>(std::stoi(ed->dir_combo->get_active_id()));
-            {
-                std::stringstream ss(ed->pin_names_entry->get_text());
-                std::istream_iterator<std::string> begin(ss);
-                std::istream_iterator<std::string> end;
-                std::vector<std::string> tags(begin, end);
-                pin->names = tags;
-            }
+
+            pin->names = ed->pin_names;
 
             Gate *gate = nullptr;
             {
@@ -859,8 +864,6 @@ void PartWizard::update_part()
     for (auto it = units.begin(); it != units.end();) {
         auto uu = it->second.uuid;
         if (units_used.count(uu) == 0) {
-            std::cout << "del sym" << std::endl;
-
             {
                 auto unit_filename = pool.get_tmp_filename(ObjectType::UNIT, uu);
                 auto fi = Gio::File::create_for_path(unit_filename);
