@@ -246,6 +246,24 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
         remote_repo = "";
     }
 
+    if (pool.get_pool_info().is_project_pool()) {
+        cache_box = PoolCacheBox::create(&appwin.app, this, pool);
+        cache_box->signal_goto().connect(sigc::mem_fun(*this, &PoolNotebook::go_to));
+        cache_box->show();
+        append_page(*cache_box, "Cache");
+        cache_box->unreference();
+        cache_box->refresh_status();
+
+        signal_switch_page().connect([this](Gtk::Widget *page, int page_num) {
+            if (in_destruction())
+                return;
+            if (page == cache_box && !cache_box->refreshed_once) {
+                cache_box->refresh_status();
+                cache_box->refreshed_once = true;
+            }
+        });
+    }
+
     construct_units();
     construct_symbols();
     construct_entities();
@@ -267,24 +285,6 @@ PoolNotebook::PoolNotebook(const std::string &bp, class PoolProjectManagerAppWin
         settings_box->unreference();
     }
 
-
-    if (pool.get_pool_info().is_project_pool()) {
-        cache_box = PoolCacheBox::create(&appwin.app, this, pool);
-        cache_box->signal_goto().connect(sigc::mem_fun(*this, &PoolNotebook::go_to));
-        cache_box->show();
-        append_page(*cache_box, "Cache");
-        cache_box->unreference();
-
-        signal_switch_page().connect([this](Gtk::Widget *page, int page_num) {
-            if (in_destruction())
-                return;
-            if (page == cache_box && !cache_box->refreshed_once) {
-                cache_box->refresh_status();
-                cache_box->refreshed_once = true;
-            }
-        });
-    }
-
     if (remote_repo.size()) {
         remote_box = PoolRemoteBox::create(*this);
 
@@ -366,7 +366,8 @@ void PoolNotebook::add_context_menu(PoolBrowser *br)
             [this, ty](const UUID &uu) {
                 auto x = get_pool_uuids(ty, uu);
                 if (auto pool2 = PoolManager::get().get_by_uuid(x.last ? x.last : x.pool)) {
-                    appwin.app.open_pool(Glib::build_filename(pool2->base_path, "pool.json"), ty, uu);
+                    auto &win = appwin.app.open_pool(Glib::build_filename(pool2->base_path, "pool.json"));
+                    win.pool_notebook_go_to(ty, uu);
                 }
             },
             [this, ty](const UUID &uu) -> bool {
@@ -524,6 +525,13 @@ void PoolNotebook::go_to(ObjectType type, const UUID &uu)
     set_current_page(page);
 }
 
+void PoolNotebook::show_settings_tab()
+{
+    auto page = page_num(*settings_box);
+    if (page != -1)
+        set_current_page(page);
+}
+
 const UUID &PoolNotebook::get_pool_uuid() const
 {
     return pool_uuid;
@@ -603,4 +611,9 @@ void PoolNotebook::install_search_once(Gtk::Widget *widget, PoolBrowser *browser
             *widget, *browser));
 }
 
+class PoolSettingsBox &PoolNotebook::get_pool_settings_box()
+{
+    return *settings_box;
+}
+
 } // namespace horizon
