@@ -3,7 +3,6 @@
 #include <glibmm/miscutils.h>
 #include <fstream>
 #include "util/util.hpp"
-#include "util/recent_util.hpp"
 #include <git2.h>
 #include <curl/curl.h>
 #include "nlohmann/json.hpp"
@@ -90,9 +89,9 @@ std::string PoolProjectManagerApplication::get_config_filename()
     return Glib::build_filename(get_config_dir(), "pool-project-manager.json");
 }
 
-void PoolProjectManagerApplication::load_from_config(const std::string &config_filename)
+void PoolProjectManagerApplication::UserConfig::load(const std::string &filename)
 {
-    json j = load_json_from_file(config_filename);
+    json j = load_json_from_file(filename);
     if (j.count("part_favorites")) {
         const json &o = j.at("part_favorites");
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
@@ -100,9 +99,42 @@ void PoolProjectManagerApplication::load_from_config(const std::string &config_f
         }
     }
     pool_doc_info_bar_dismissed = j.value("pool_doc_info_bar_dismissed", false);
-    recent_from_json(recent_items, j);
+    project_base_path = j.value("project_base_path", "");
+    project_author = j.value("project_author", "");
+    if (j.count("recent")) {
+        const json &o = j["recent"];
+        for (const auto &[fn, v] : o.items()) {
+            if (Glib::file_test(fn, Glib::FILE_TEST_IS_REGULAR))
+                recent_items.emplace(fn, Glib::DateTime::create_now_local(v.get<int64_t>()));
+        }
+    }
 }
 
+void PoolProjectManagerApplication::UserConfig::save(const std::string &filename)
+{
+    json j;
+    {
+        json k;
+        for (const auto &it : part_favorites) {
+            k.push_back((std::string)it);
+        }
+        j["part_favorites"] = k;
+    }
+    for (const auto &it : recent_items) {
+        j["recent"][it.first] = it.second.to_unix();
+    }
+    j["project_author"] = project_author;
+    j["project_base_path"] = project_base_path;
+    j["pool_doc_info_bar_dismissed"] = pool_doc_info_bar_dismissed;
+    save_json_to_file(filename, j);
+}
+
+void PoolProjectManagerApplication::UserConfig::add_recent_item(const std::string &path)
+{
+    recent_items[path] = Glib::DateTime::create_now_local();
+}
+
+
 void PoolProjectManagerApplication::on_startup()
 {
     // Call the base class's implementation.
@@ -120,9 +152,8 @@ void PoolProjectManagerApplication::on_startup()
     add_action("view_log", [this] { show_log_window(); });
     set_accel_for_action("app.quit", "<Ctrl>Q");
 
-    recent_items.clear();
     if (Glib::file_test(get_config_filename(), Glib::FILE_TEST_IS_REGULAR)) {
-        load_from_config(get_config_filename());
+        user_config.load(get_config_filename());
     }
     else {
         std::vector<std::string> old_configs;
@@ -130,7 +161,7 @@ void PoolProjectManagerApplication::on_startup()
         old_configs.push_back(Glib::build_filename(get_config_dir(), "project-manager.json"));
         for (const auto &it : old_configs) {
             if (Glib::file_test(it, Glib::FILE_TEST_IS_REGULAR))
-                load_from_config(it);
+                user_config.load(it);
         }
     }
 
@@ -196,21 +227,7 @@ LogWindow *PoolProjectManagerApplication::show_log_window(guint32 timestamp)
 
 void PoolProjectManagerApplication::on_shutdown()
 {
-    auto config_filename = get_config_filename();
-
-    json j;
-    {
-        json k;
-        for (const auto &it : part_favorites) {
-            k.push_back((std::string)it);
-        }
-        j["part_favorites"] = k;
-    }
-    for (const auto &it : recent_items) {
-        j["recent"][it.first] = it.second.to_unix();
-    }
-    j["pool_doc_info_bar_dismissed"] = pool_doc_info_bar_dismissed;
-    save_json_to_file(config_filename, j);
+    user_config.save(get_config_filename());
 
     StockInfoProvider::cleanup();
 }
@@ -221,22 +238,29 @@ void PoolProjectManagerApplication::on_open(const Gio::Application::type_vec_fil
 
     // The application has been asked to open some files,
     // so let's open a new view for each one.
-    PoolProjectManagerAppWindow *appwindow = nullptr;
-    auto windows = get_windows();
-    if (windows.size() > 0)
-        appwindow = dynamic_cast<PoolProjectManagerAppWindow *>(windows[0]);
 
-    if (!appwindow)
-        appwindow = create_appwindow();
-
-    for (const auto &file : files)
-        appwindow->open_file_view(file);
-
-    appwindow->present();
+    for (const auto &file : files) {
+        if (!present_existing_window(file->get_path())) { // not opened anywhere
+            auto appwindow = create_appwindow();
+            appwindow->open_file_view(file);
+            appwindow->present();
+        }
+    }
 }
 
-void PoolProjectManagerApplication::open_pool(const std::string &pool_json, ObjectType type, const UUID &uu,
-                                              guint32 timestamp)
+bool PoolProjectManagerApplication::present_existing_window(const std::string &path)
+{
+    auto windows = dynamic_cast_vector<PoolProjectManagerAppWindow *>(get_windows());
+    for (auto &win : windows) {
+        if (win->get_filename() == path) {
+            win->present();
+            return true;
+        }
+    }
+    return false;
+}
+
+PoolProjectManagerAppWindow &PoolProjectManagerApplication::open_pool(const std::string &pool_json, guint32 timestamp)
 {
     PoolProjectManagerAppWindow *appwindow = nullptr;
     for (auto ws : get_windows()) {
@@ -248,8 +272,9 @@ void PoolProjectManagerApplication::open_pool(const std::string &pool_json, Obje
     }
     if (!appwindow)
         appwindow = create_appwindow();
-    appwindow->open_pool(pool_json, type, uu);
+    appwindow->open_pool(pool_json);
     appwindow->present(timestamp);
+    return *appwindow;
 }
 
 void PoolProjectManagerApplication::on_hide_window(Gtk::Window *window)
@@ -319,7 +344,7 @@ bool PoolProjectManagerApplication::close_windows(std::vector<CloseOrHomeWindow>
         }
     }
     else {
-        for (auto win : windows) {
+        for (auto &[filenanme, win] : files_windows) {
             win.win.prepare_close();
             auto procs = win.win.get_processes();
             for (auto &it : procs) {
