@@ -19,21 +19,33 @@ protected:
 
 public:
     static Glib::RefPtr<PoolProjectManagerApplication> create();
-    std::string get_config_filename();
     const std::string &get_ep_broadcast() const;
     void send_json(int pid, const json &j);
     zmq::context_t zctx;
 
-    std::map<std::string, Glib::DateTime> recent_items;
+    class UserConfig {
+    private:
+        friend PoolProjectManagerApplication;
+        UserConfig() = default;
+        void load(const std::string &filename);
+        void save(const std::string &filename);
 
-    std::deque<UUID> part_favorites;
-    bool pool_doc_info_bar_dismissed = false;
+    public:
+        void add_recent_item(const std::string &path);
+
+        std::map<std::string, Glib::DateTime> recent_items;
+        std::deque<UUID> part_favorites;
+        bool pool_doc_info_bar_dismissed = false;
+        std::string project_author;
+        std::string project_base_path;
+    };
+
+    UserConfig user_config;
 
     void close_appwindows(std::set<Gtk::Window *> wins);
     Preferences &get_preferences();
 
-    void open_pool(const std::string &pool_json, ObjectType type = ObjectType::INVALID, const UUID &uu = UUID(),
-                   guint32 timestamp = 0);
+    class PoolProjectManagerAppWindow &open_pool(const std::string &pool_json, guint32 timestamp = 0);
 
     class PreferencesWindow *show_preferences_window(guint32 timestamp = 0);
     class PoolsWindow *show_pools_window(guint32 timestamp = 0);
@@ -57,6 +69,8 @@ public:
     };
     bool close_windows(std::vector<CloseOrHomeWindow> windows);
 
+    bool present_existing_window(const std::string &path);
+
 protected:
     // Override default signal handlers:
     void on_activate() override;
@@ -73,7 +87,6 @@ private:
     void on_action_quit();
     void on_action_new_window();
     void on_action_about();
-    void load_from_config(const std::string &config_filename);
     Preferences preferences;
     class PreferencesWindow *preferences_window = nullptr;
     class PoolsWindow *pools_window = nullptr;
@@ -84,6 +97,8 @@ private:
     type_signal_pool_items_edited s_signal_pool_items_edited;
     type_signal_pool_updated s_signal_pool_updated;
 
+    std::string get_config_filename();
+
 public:
     const UUID ipc_cookie;
     zmq::socket_t sock_broadcast;
