@@ -7,9 +7,9 @@
 #include "pool/pool_manager.hpp"
 #include "pool/project_pool.hpp"
 #include "pool-mgr/pool_notebook.hpp"
+#include "pool-mgr/pool_settings_box.hpp"
 #include "util/pool_check_schema_update.hpp"
 #include "util/gtk_util.hpp"
-#include "util/recent_util.hpp"
 #include "widgets/recent_item_box.hpp"
 #include "nlohmann/json.hpp"
 #include "pool-mgr/editors/editor_window.hpp"
@@ -31,6 +31,8 @@
 #include "blocks/blocks_schematic.hpp"
 #include "project/project.hpp"
 #include <filesystem>
+#include "util/fs_util.hpp"
+#include "widgets/sqlite_shell.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -127,7 +129,7 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
     info_bar_pool_doc->signal_response().connect([this](int resp) {
         if (resp == Gtk::RESPONSE_OK) {
             info_bar_hide(info_bar_pool_doc);
-            app.pool_doc_info_bar_dismissed = true;
+            app.user_config.pool_doc_info_bar_dismissed = true;
         }
     });
     info_bar_hide(info_bar_pool_doc);
@@ -216,6 +218,19 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
         return false;
     });
 
+    signal_key_press_event().connect([this](const GdkEventKey *ev) {
+        if (ev->keyval == GDK_KEY_S && (ev->state & GDK_CONTROL_MASK)) {
+            if (pool || project) {
+                auto w = new SQLiteShellWindow(get_pool_base_path() + "/pool.db");
+                w->set_transient_for(*this);
+                w->signal_hide().connect([w] { delete w; });
+                w->present();
+                return true;
+            }
+        }
+        return false;
+    });
+
     pool_update_dispatcher.connect(sigc::mem_fun(*this, &PoolProjectManagerAppWindow::handle_pool_update_progress));
     app.signal_pool_items_edited().connect(
             sigc::track_obj([this](const auto &filenames) { pool_update(filenames); }, *this));
@@ -223,8 +238,12 @@ PoolProjectManagerAppWindow::PoolProjectManagerAppWindow(BaseObjectType *cobject
             [this](const auto &path) {
                 if (part_browser_window)
                     part_browser_window->pool_updated(path);
+                if (project && project->pool_directory == path)
+                    view_project.update_pools_label();
             },
             *this));
+
+    check_mtimes_dispatcher.connect([this] { pool_update(); });
 }
 
 
@@ -323,6 +342,14 @@ void PoolProjectManagerAppWindow::pool_update(const std::vector<std::string> &fi
 
     set_pool_updating(true, true);
     pool_update_n_files = 0;
+    if (pool_update_n_files_last == 0) { // use estimate
+        Pool my_pool(get_pool_base_path());
+        SQLite::Query q(my_pool.db,
+                        "SELECT SUM(x) FROM (SELECT COUNT(*) AS x FROM parts WHERE parametric_table != '' "
+                        "UNION ALL SELECT COUNT(*) AS x FROM all_items_view)");
+        if (q.step())
+            pool_update_n_files_last = q.get<int>(0);
+    }
     pool_updating = true;
     pool_update_filenames = filenames;
     pool_update_status_queue.clear();
@@ -600,7 +627,8 @@ json PoolProjectManagerAppWindow::handle_req(const json &j)
             pool_path = pool2->base_path;
         }
         if (pool_path.size()) {
-            app.open_pool(Glib::build_filename(pool_path, "pool.json"), type, uu, timestamp);
+            auto &win = app.open_pool(Glib::build_filename(pool_path, "pool.json"), timestamp);
+            win.pool_notebook_go_to(type, uu);
         }
     }
     else if (op == "backannotate") {
@@ -657,6 +685,8 @@ void PoolProjectManagerAppWindow::set_pool_updating(bool v, bool success)
                     500);
         }
     }
+    if (!get_realized())
+        return;
     auto win = get_window();
     if (v)
         win->set_cursor(Gdk::Cursor::create(win->get_display(), "progress"));
@@ -754,6 +784,7 @@ void PoolProjectManagerAppWindow::handle_new_project()
     if (!check_pools())
         return;
     view_create_project.clear();
+    view_create_project.focus();
     set_view_mode(ViewMode::CREATE_PROJECT);
 }
 
@@ -765,18 +796,18 @@ void PoolProjectManagerAppWindow::handle_new_pool()
 void PoolProjectManagerAppWindow::handle_create()
 {
     if (view_mode == ViewMode::CREATE_PROJECT) {
-        auto r = view_create_project.create();
-        if (r.first) {
+        const auto r = view_create_project.create();
+        if (r) {
             view_create_project.clear();
-            open_file_view(Gio::File::create_for_path(r.second));
+            open_file_view(Gio::File::create_for_path(*r));
         }
     }
     else if (view_mode == ViewMode::CREATE_POOL) {
-        auto r = view_create_pool.create();
-        if (r.first) {
+        const auto r = view_create_pool.create();
+        if (r) {
             view_create_pool.clear();
-            PoolManager::get().add_pool(Glib::path_get_dirname(r.second));
-            open_file_view(Gio::File::create_for_path(r.second));
+            PoolManager::get().add_pool(Glib::path_get_dirname(*r));
+            open_file_view(Gio::File::create_for_path(*r));
         }
     }
 }
@@ -803,6 +834,20 @@ static std::optional<std::string> peek_name(const std::string &path)
     return name;
 }
 
+static std::vector<std::pair<std::string, Glib::DateTime>>
+recent_sort(const std::map<std::string, Glib::DateTime> &recent_items)
+{
+    std::vector<std::pair<std::string, Glib::DateTime>> recent_items_sorted;
+
+    recent_items_sorted.reserve(recent_items.size());
+    for (const auto &it : recent_items) {
+        recent_items_sorted.emplace_back(it.first, it.second);
+    }
+    std::sort(recent_items_sorted.begin(), recent_items_sorted.end(),
+              [](const auto &a, const auto &b) { return a.second.to_unix() > b.second.to_unix(); });
+    return recent_items_sorted;
+}
+
 void PoolProjectManagerAppWindow::update_recent_items()
 {
     {
@@ -813,7 +858,7 @@ void PoolProjectManagerAppWindow::update_recent_items()
             }
         }
     }
-    std::vector<std::pair<std::string, Glib::DateTime>> recent_items_sorted = recent_sort(app.recent_items);
+    std::vector<std::pair<std::string, Glib::DateTime>> recent_items_sorted = recent_sort(app.user_config.recent_items);
     for (const auto &it : recent_items_sorted) {
         const std::string &path = it.first;
         if (const auto name = peek_name(path)) {
@@ -824,7 +869,7 @@ void PoolProjectManagerAppWindow::update_recent_items()
                 recent_projects_listbox->append(*box);
             box->show();
             box->signal_remove().connect([this, box] {
-                app.recent_items.erase(box->path);
+                app.user_config.recent_items.erase(box->path);
                 Glib::signal_idle().connect_once([this] { update_recent_items(); });
             });
         }
@@ -876,7 +921,7 @@ bool PoolProjectManagerAppWindow::really_close_pool_or_project()
         }
         if (pool_notebook)
             pool_notebook->prepare_close();
-        app.recent_items[Glib::build_filename(pool->get_base_path(), "pool.json")] = Glib::DateTime::create_now_local();
+        app.user_config.add_recent_item(Glib::build_filename(pool->get_base_path(), "pool.json"));
         delete pool_notebook;
         info_bar_hide(info_bar_pool_not_added);
         pool_notebook = nullptr;
@@ -890,7 +935,7 @@ bool PoolProjectManagerAppWindow::really_close_pool_or_project()
             md.run();
             return false;
         }
-        app.recent_items[project_filename] = Glib::DateTime::create_now_local();
+        app.user_config.add_recent_item(project_filename);
         info_bar_hide(info_bar_gitignore);
         project.reset();
         cleanup();
@@ -900,6 +945,8 @@ bool PoolProjectManagerAppWindow::really_close_pool_or_project()
         set_view_mode(ViewMode::OPEN);
     }
     output_window->clear_all();
+    if (check_mtimes_thread.joinable())
+        check_mtimes_thread.join();
     return true;
 }
 
@@ -935,7 +982,7 @@ void PoolProjectManagerAppWindow::set_view_mode(ViewMode mode)
         button_update->show();
         hamburger_menu_button->show();
         header->set_title("Pool manager");
-        if (!app.pool_doc_info_bar_dismissed)
+        if (!app.user_config.pool_doc_info_bar_dismissed)
             info_bar_show(info_bar_pool_doc);
         break;
 
@@ -987,6 +1034,12 @@ void PoolProjectManagerAppWindow::pool_notebook_go_to(ObjectType type, const UUI
         pool_notebook->go_to(type, uu);
 }
 
+void PoolProjectManagerAppWindow::pool_notebook_show_settings_tab()
+{
+    if (pool_notebook)
+        pool_notebook->show_settings_tab();
+}
+
 PoolProjectManagerAppWindow *PoolProjectManagerAppWindow::create(PoolProjectManagerApplication &app)
 {
     // Load the Builder file and instantiate its widgets.
@@ -1003,17 +1056,21 @@ PoolProjectManagerAppWindow *PoolProjectManagerAppWindow::create(PoolProjectMana
     return window;
 }
 
-void PoolProjectManagerAppWindow::check_schema_update(const std::string &base_path)
+bool PoolProjectManagerAppWindow::check_schema_update(const std::string &base_path)
 {
     const auto r = pool_check_schema_update(base_path, *this);
     if (r == CheckSchemaUpdateResult::INSTALLATION_UUID_MISMATCH)
         info_bar_show(info_bar_installation_uuid_mismatch);
-    if (r != CheckSchemaUpdateResult::NO_UPDATE)
+
+    if (r != CheckSchemaUpdateResult::NO_UPDATE) {
         app.signal_pool_updated().emit(base_path);
+        return true;
+    }
+    return false;
 }
 
 
-void PoolProjectManagerAppWindow::check_pool_update(const std::string &base_path)
+static bool check_pool_update_inc(const std::string &base_path)
 {
     Pool my_pool(base_path);
     for (auto &[bp, uu] : my_pool.get_actually_included_pools(false)) {
@@ -1029,8 +1086,9 @@ void PoolProjectManagerAppWindow::check_pool_update(const std::string &base_path
                                 "SELECT a.time > b.time FROM inc.last_updated AS a LEFT JOIN last_updated AS b");
                 if (q.step()) {
                     if (q.get<int>(0)) {
-                        pool_update();
-                        return;
+                        Logger::log_info("pool " + base_path + " got updated", Logger::Domain::POOL_UPDATE,
+                                         "beacause included pool " + bp + " has been updated");
+                        return true;
                     }
                 }
             }
@@ -1038,6 +1096,7 @@ void PoolProjectManagerAppWindow::check_pool_update(const std::string &base_path
             my_pool.db.execute("DETACH inc");
         }
     }
+    return false;
 }
 
 
@@ -1137,18 +1196,70 @@ gboolean PoolProjectManagerAppWindow::part_browser_key_pressed(GtkEventControlle
 }
 #endif
 
+void PoolProjectManagerAppWindow::check_mtimes(const std::string &path)
+{
+    try {
+        Pool my_pool(path);
+        int64_t last_update_time = 0;
+        {
+            SQLite::Query q(my_pool.db, "SELECT time FROM last_updated");
+            if (q.step())
+                last_update_time = q.get<sqlite3_int64>(0);
+            else
+                return;
+        }
+
+        for (auto &[bp, uu] : my_pool.get_actually_included_pools(true)) {
+            PoolInfo pool_info(bp);
+            for (const auto &[type, name] : IPool::type_names) {
+                const auto p = fs::u8path(bp) / name;
+                if (!fs::is_directory(p))
+                    continue;
+                for (const auto &ent : fs::recursive_directory_iterator(p)) {
+                    if (!ent.is_regular_file())
+                        continue;
+                    if (endswith(ent.path().filename().u8string(), ".json")) {
+                        const auto mtime = ent.last_write_time().time_since_epoch().count();
+                        if (mtime > last_update_time) {
+                            Logger::log_info("pool " + path + " got updated", Logger::Domain::POOL_UPDATE,
+                                             "beacause " + ent.path().u8string() + " has been modified");
+                            check_mtimes_dispatcher.emit();
+                            return;
+                        }
+                    }
+                }
+            }
+        }
+    }
+    catch (const std::exception &e) {
+        Logger::log_critical("exception thrown in check_mtimes", Logger::Domain::POOL_UPDATE, e.what());
+    }
+    catch (...) {
+        Logger::log_critical("unknown exception thrown in check_mtimes", Logger::Domain::POOL_UPDATE);
+    }
+}
+
+
+void PoolProjectManagerAppWindow::check_pool_update(const std::string &base_path)
+{
+    if (check_schema_update(base_path)) {
+        // it's okay, pool got updated
+    }
+    else if (check_pool_update_inc(base_path)) {
+        pool_update();
+    }
+    else {
+        check_mtimes_thread = std::thread(&PoolProjectManagerAppWindow::check_mtimes, this, base_path);
+    }
+}
+
 void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &file)
 {
     auto path = file->get_path();
-    app.recent_items[path] = Glib::DateTime::create_now_local();
+    app.user_config.add_recent_item(path);
 
-    auto windows = dynamic_cast_vector<PoolProjectManagerAppWindow *>(app.get_windows());
-    for (auto &win : windows) {
-        if (win->get_filename() == path) {
-            win->present();
-            return;
-        }
-    }
+    if (app.present_existing_window(path))
+        return;
 
     pool_update_n_files_last = 0;
 
@@ -1159,24 +1270,21 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
                 throw std::runtime_error("pool.json not found");
             }
             auto pool_base_path = file->get_parent()->get_path();
-            check_schema_update(pool_base_path);
+            check_pool_update(pool_base_path);
             set_view_mode(ViewMode::POOL);
             pool_notebook = new PoolNotebook(pool_base_path, *this);
             if (!PoolManager::get().get_pools().count(pool_base_path) && !pool->get_pool_info().is_project_pool()) {
                 info_bar_show(info_bar_pool_not_added);
             }
-            pool_notebook->signal_saved().connect([this] { set_version_info(""); });
             pool_box->pack_start(*pool_notebook, true, true, 0);
             pool_notebook->show();
             header->set_subtitle(pool_base_path);
-            const auto &version = pool->get_pool_info().version;
-            const auto version_msg = version.get_message(ObjectType::POOL);
-            if (version_msg.size()) {
-                set_version_info(version_msg
-                                 + " This only applies to the pool.json file. Pool items have different versions.");
-            }
-            else {
-                set_version_info("");
+
+            {
+                auto &settings_box = pool_notebook->get_pool_settings_box();
+                set_version_info(settings_box.get_version_message());
+                settings_box.signal_changed().connect(
+                        [this, &settings_box] { set_version_info(settings_box.get_version_message()); });
             }
 
             if (pool->get_pool_info().is_project_pool()) {
@@ -1187,7 +1295,7 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
                         if (w->get_view_mode() == PoolProjectManagerAppWindow::ViewMode::PROJECT) {
                             const auto win_path = Glib::path_get_dirname(w->get_filename());
                             if (win_path == project_path) {
-                                set_title(w->get_project_title() + " - Pool");
+                                set_title(w->get_project_title() + " – Pool");
                                 break;
                             }
                         }
@@ -1211,8 +1319,6 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         project_filename = path;
         project_needs_save = false;
 
-        check_schema_update(project->pool_directory);
-
         check_pool_update(project->pool_directory);
 
         {
@@ -1246,7 +1352,7 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
 
         view_project.reset_pool_cache_status();
 
-        part_browser_window = PartBrowserWindow::create(this, project->pool_directory, app.part_favorites);
+        part_browser_window = PartBrowserWindow::create(this, project->pool_directory, app.user_config.part_favorites);
         part_browser_window->signal_place_part().connect(
                 sigc::mem_fun(*this, &PoolProjectManagerAppWindow::handle_place_part));
         part_browser_window->signal_assign_part().connect(
@@ -1263,6 +1369,7 @@ void PoolProjectManagerAppWindow::open_file_view(const Glib::RefPtr<Gio::File> &
         set_view_mode(ViewMode::PROJECT);
 
         view_project.update_meta();
+        view_project.update_pools_label();
     }
 }
 
@@ -1273,14 +1380,11 @@ const std::string &PoolProjectManagerAppWindow::get_project_title() const
     return project_title;
 }
 
-void PoolProjectManagerAppWindow::open_pool(const std::string &pool_json, ObjectType type, const UUID &uu)
+void PoolProjectManagerAppWindow::open_pool(const std::string &pool_json)
 {
     open_file_view(Gio::File::create_for_path(pool_json));
-    if (type != ObjectType::INVALID)
-        pool_notebook_go_to(type, uu);
 }
 
-
 void PoolProjectManagerAppWindow::handle_place_part(const UUID &uu)
 {
     if (auto proc = find_top_schematic_process()) {
@@ -1316,8 +1420,9 @@ std::string PoolProjectManagerAppWindow::get_pool_base_path() const
         throw std::runtime_error("can't locate pool");
 }
 
-PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManagerProcess::Type type,
-                                                              const std::vector<std::string> &args, SpawnFlags flags)
+PoolProjectManagerAppWindow::SpawnResult PoolProjectManagerAppWindow::spawn(PoolProjectManagerProcess::Type type,
+                                                                            const std::vector<std::string> &args,
+                                                                            SpawnFlags flags)
 {
     const auto pool_base_path = get_pool_base_path();
 
@@ -1335,12 +1440,12 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
                                       Gtk::BUTTONS_OK);
                 md.set_secondary_text("Try updating the pool");
                 md.run();
-                return nullptr;
+                return {nullptr, false};
             }
         }
 
         if (!check_autosave(type, args))
-            return nullptr;
+            return {nullptr, false};
 
         auto uu = UUID::random();
         const bool read_only = static_cast<int>(flags) & static_cast<int>(SpawnFlags::READ_ONLY);
@@ -1378,7 +1483,7 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
         if (proc.win)
             proc.win->signal_saved().connect([this](auto fn) { s_signal_process_saved.emit(fn); });
 
-        return &proc;
+        return {&proc, true};
     }
     else { // present imp
         auto proc = find_process(args.at(0));
@@ -1390,7 +1495,7 @@ PoolProjectManagerProcess *PoolProjectManagerAppWindow::spawn(PoolProjectManager
         else {
             proc->win->present();
         }
-        return proc;
+        return {proc, false};
     }
 }
 
