@@ -9,6 +9,7 @@
 #include "blocks/blocks.hpp"
 #include "pool/pool_cache_status.hpp"
 #include "util/gtk_util.hpp"
+#include "pool/pool.hpp"
 
 namespace horizon {
 
@@ -34,9 +35,17 @@ PoolProjectManagerViewCreateProject::PoolProjectManagerViewCreateProject(const G
 void PoolProjectManagerViewCreateProject::clear()
 {
     meta_editor->clear();
-    meta_editor->preset();
+    meta_editor->preset(win.app.user_config.project_author);
+    meta_editor->set_use_automatic_name();
     project_dir_label->set_text("");
     project_path_chooser->unselect_all();
+    if (win.app.user_config.project_base_path.size())
+        project_path_chooser->set_filename(win.app.user_config.project_base_path);
+}
+
+void PoolProjectManagerViewCreateProject::focus()
+{
+    meta_editor->focus_title();
 }
 
 void PoolProjectManagerViewCreateProject::populate_pool_combo()
@@ -49,34 +58,33 @@ void PoolProjectManagerViewCreateProject::populate_pool_combo()
     project_pool_combo->set_active(0);
 }
 
-std::pair<bool, std::string> PoolProjectManagerViewCreateProject::create()
+std::optional<std::string> PoolProjectManagerViewCreateProject::create()
 {
-    bool r = false;
-    std::string s;
     try {
         Project prj(UUID::random());
-        prj.base_path =
-                Glib::build_filename(project_path_chooser->get_file()->get_path(), meta_values.at("project_name"));
+        const auto base_path = project_path_chooser->get_file()->get_path();
+        prj.base_path = Glib::build_filename(base_path, meta_values.at("project_name"));
         auto pool_uuid = static_cast<std::string>(project_pool_combo->get_active_id());
-
-        s = prj.create(meta_values, pool_uuid, PoolManager::get().get_by_uuid(pool_uuid)->default_via);
-        r = true;
+        auto pool = PoolManager::get().get_by_uuid(pool_uuid);
+        if (!pool)
+            throw std::runtime_error("pool not found");
+        win.app.user_config.project_author = meta_values.at("author");
+        win.app.user_config.project_base_path = base_path;
+        return prj.create(meta_values, *pool);
     }
     catch (const std::exception &e) {
-        r = false;
         Gtk::MessageDialog md(win, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
                               Gtk::BUTTONS_OK);
         md.set_secondary_text(e.what());
         md.run();
     }
     catch (const Glib::Error &e) {
-        r = false;
         Gtk::MessageDialog md(win, "Error creating project", false /* use_markup */, Gtk::MESSAGE_ERROR,
                               Gtk::BUTTONS_OK);
         md.set_secondary_text(e.what());
         md.run();
     }
-    return {r, s};
+    return {};
 }
 
 void PoolProjectManagerViewCreateProject::update()
@@ -101,6 +109,39 @@ void PoolProjectManagerViewCreateProject::update()
     signal_valid_change().emit(valid);
 }
 
+class OpeningSpinner : public Gtk::Revealer {
+public:
+    OpeningSpinner();
+    void set_active(bool a);
+
+private:
+    Gtk::Spinner *spinner = nullptr;
+};
+
+OpeningSpinner::OpeningSpinner()
+{
+    set_transition_type(Gtk::REVEALER_TRANSITION_TYPE_CROSSFADE);
+    auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 4));
+    spinner = Gtk::manage(new Gtk::Spinner());
+    box->pack_start(*spinner, false, false, 0);
+
+    auto la = Gtk::manage(new Gtk::Label("Opening…"));
+    box->pack_start(*la, false, false, 0);
+
+    box->show_all();
+    add(*box);
+}
+
+void OpeningSpinner::set_active(bool a)
+
+{
+    set_reveal_child(a);
+    if (a)
+        spinner->start();
+    else
+        spinner->stop();
+}
+
 PoolProjectManagerViewProject::PoolProjectManagerViewProject(const Glib::RefPtr<Gtk::Builder> &builder,
                                                              PoolProjectManagerAppWindow &w)
     : win(w)
@@ -112,8 +153,24 @@ PoolProjectManagerViewProject::PoolProjectManagerViewProject(const Glib::RefPtr<
     builder->get_widget("label_project_title", label_project_title);
     builder->get_widget("label_project_author", label_project_author);
     builder->get_widget("label_project_directory", label_project_directory);
+    builder->get_widget("label_project_pools", label_project_pools);
     builder->get_widget("pool_cache_status_label", pool_cache_status_label);
 
+    {
+        Gtk::Box *board_opening_box;
+        builder->get_widget("board_opening_box", board_opening_box);
+        board_spinner = Gtk::manage(new OpeningSpinner());
+        board_opening_box->pack_start(*board_spinner, false, false, 0);
+        board_spinner->show();
+    }
+    {
+        Gtk::Box *schematic_opening_box;
+        builder->get_widget("schematic_opening_box", schematic_opening_box);
+        schematic_spinner = Gtk::manage(new OpeningSpinner());
+        schematic_opening_box->pack_start(*schematic_spinner, false, false, 0);
+        schematic_spinner->show();
+    }
+
     Gtk::Button *open_button;
     builder->get_widget("prj_open_dir_button", open_button);
     open_button->signal_clicked().connect([this] { open_directory(win, win.project_filename); });
@@ -134,6 +191,14 @@ PoolProjectManagerViewProject::PoolProjectManagerViewProject(const Glib::RefPtr<
         }
     });
 
+    Gtk::Button *change_button;
+    builder->get_widget("prj_change_pools_button", change_button);
+    change_button->signal_clicked().connect([this] {
+        auto path = Glib::build_filename(win.project->pool_directory, "pool.json");
+        auto &win2 = win.app.open_pool(path);
+        win2.pool_notebook_show_settings_tab();
+    });
+
     button_top_schematic->signal_clicked().connect(
             sigc::mem_fun(*this, &PoolProjectManagerViewProject::open_top_schematic));
     button_board->signal_clicked().connect(sigc::mem_fun(*this, &PoolProjectManagerViewProject::open_board));
@@ -147,14 +212,24 @@ void PoolProjectManagerViewProject::open_top_schematic()
 {
     auto prj = win.project.get();
     std::vector<std::string> args = {prj->blocks_filename, prj->pictures_directory};
-    win.spawn(PoolProjectManagerProcess::Type::IMP_SCHEMATIC, args);
+    auto proc = win.spawn(PoolProjectManagerProcess::Type::IMP_SCHEMATIC, args);
+    if (proc.spawned) {
+        schematic_spinner->set_active(true);
+        proc.proc->signal_exited().connect([this](int, bool) { schematic_spinner->set_active(false); });
+        proc.proc->signal_ready().connect([this] { schematic_spinner->set_active(false); });
+    }
 }
 
 void PoolProjectManagerViewProject::open_board()
 {
     auto prj = win.project.get();
     std::vector<std::string> args = {prj->board_filename, prj->blocks_filename, prj->pictures_directory};
-    win.spawn(PoolProjectManagerProcess::Type::IMP_BOARD, args);
+    auto proc = win.spawn(PoolProjectManagerProcess::Type::IMP_BOARD, args);
+    if (proc.spawned) {
+        board_spinner->set_active(true);
+        proc.proc->signal_exited().connect([this](int, bool) { board_spinner->set_active(false); });
+        proc.proc->signal_ready().connect([this] { board_spinner->set_active(false); });
+    }
 }
 
 void PoolProjectManagerViewProject::handle_button_part_browser()
@@ -178,7 +253,7 @@ bool PoolProjectManagerViewProject::update_meta()
     if (meta.count("author"))
         author = meta.at("author");
     if (title.size())
-        win.set_title(title + " - Horizon EDA");
+        win.set_title(title + " – Horizon EDA");
     else
         win.set_title("Project Manager");
     label_project_title->set_text(title);
@@ -187,6 +262,26 @@ bool PoolProjectManagerViewProject::update_meta()
     return meta.size();
 }
 
+void PoolProjectManagerViewProject::update_pools_label()
+{
+    Pool my_pool(win.project->pool_directory);
+    std::string pools_label;
+    std::string pools_tooltip;
+    for (const auto &[bp, uu] : my_pool.get_actually_included_pools(false)) {
+        const auto inc_pool = PoolManager::get().get_by_uuid(uu);
+        if (inc_pool) {
+            if (pools_label.size()) {
+                pools_label += ", ";
+                pools_tooltip += "\n";
+            }
+            pools_label += inc_pool->name;
+            pools_tooltip += inc_pool->name;
+        }
+    }
+    label_project_pools->set_text(pools_label);
+    label_project_pools->set_tooltip_text(pools_tooltip);
+}
+
 void PoolProjectManagerViewProject::update_pool_cache_status(const PoolCacheStatus &status)
 {
     std::string txt;
