@@ -12,6 +12,7 @@
 #include "util/str_util.hpp"
 #include "blocks/blocks_schematic.hpp"
 #include "logger/logger.hpp"
+#include "pool/pool_info.hpp"
 
 namespace horizon {
 
@@ -98,8 +99,7 @@ static const std::vector<std::string> gitignore_lines = {
         "pool/*.db", "pool/*.db-*", "*.imp_meta", "*.autosave", "*.bak",
 };
 
-std::string Project::create(const std::map<std::string, std::string> &meta, const UUID &pool_uuid,
-                            const UUID &default_via)
+std::string Project::create(const std::map<std::string, std::string> &meta, const PoolInfo &pool)
 {
     if (Glib::file_test(base_path, Glib::FILE_TEST_EXISTS)) {
         throw std::runtime_error("project directory already exists");
@@ -118,6 +118,10 @@ std::string Project::create(const std::map<std::string, std::string> &meta, cons
     auto &top_block = blocks.get_top_block_item();
 
     top_block.block.project_meta = meta;
+    {
+        auto &sheet = top_block.schematic.sheets.begin()->second;
+        sheet.frame.uuid = pool.default_frame;
+    }
     const auto top_block_filename = Glib::build_filename(blocks.base_path, top_block.block_filename);
     const auto top_sch_filename = Glib::build_filename(blocks.base_path, top_block.schematic_filename);
     save_json_to_file(top_block_filename, top_block.block.serialize());
@@ -130,7 +134,7 @@ std::string Project::create(const std::map<std::string, std::string> &meta, cons
 
     pictures_directory = Glib::build_filename(base_path, "pictures");
     pool_cache_directory_old = Glib::build_filename(base_path, "cache");
-    pool_uuid_old = pool_uuid;
+    pool_uuid_old = pool.uuid;
     pool_directory = Glib::build_filename(base_path, "pool");
     {
         mkdir_if_not_exists(pool_directory, false);
@@ -138,17 +142,17 @@ std::string Project::create(const std::map<std::string, std::string> &meta, cons
         info.uuid = PoolInfo::project_pool_uuid;
         info.name = "Project pool";
         info.base_path = pool_directory;
-        info.pools_included = {pool_uuid};
+        info.pools_included = {pool.uuid};
         info.save();
         ProjectPool::create_directories(pool_directory);
     }
 
     Board board(UUID::random(), top_block.block);
-    if (default_via) {
+    if (pool.default_via) {
         auto &rule_via = board.rules.add_rule_t<RuleVia>();
-        rule_via.padstack = default_via;
+        rule_via.padstack = pool.default_via;
     }
-    board.fab_output_settings.prefix = name;
+    board.gerber_output_settings.prefix = name;
 
     board_filename = Glib::build_filename(base_path, "board.json");
     save_json_to_file(board_filename, board.serialize());
