@@ -1,7 +1,6 @@
 #pragma once
 #include <Python.h>
-#include "block/block.hpp"
-#include "schematic/schematic.hpp"
+#include "blocks/blocks_schematic.hpp"
 #include "project/project.hpp"
 #include "pool/project_pool.hpp"
 
@@ -9,10 +8,9 @@ extern PyTypeObject SchematicType;
 
 class SchematicWrapper {
 public:
-    SchematicWrapper(const horizon::Project &prj, const horizon::UUID &block_uuid);
+    SchematicWrapper(const horizon::Project &pr);
     horizon::ProjectPool pool;
-    horizon::Block block;
-    horizon::Schematic schematic;
+    horizon::BlocksSchematic blocks;
 };
 
 typedef struct {
