@@ -3,6 +3,7 @@
 #include "pool/entity.hpp"
 #include <set>
 #include <forward_list>
+#include <iostream>
 #include "nlohmann/json.hpp"
 #include "util/util.hpp"
 #include "logger/logger.hpp"
@@ -49,7 +50,7 @@ json Schematic::Annotation::serialize() const
     return j;
 }
 
-static const unsigned int app_version = 3;
+static const unsigned int app_version = 6;
 
 unsigned int Schematic::get_app_version()
 {
@@ -771,6 +772,15 @@ void Schematic::expand_connectivity(bool careful)
                     sheet.warnings.emplace_back(pin_pos, "Pin on pin");
                 if (junction_coords.count(pin_pos))
                     sheet.warnings.emplace_back(pin_pos, "Pin on junction");
+                if (it_pin.second.connection_count == 0) {
+                    const auto path = UUIDPath<2>(schsym.gate->uuid, it_pin.second.uuid);
+                    if (schsym.component->connections.count(path)) {
+                        auto &c = schsym.component->connections.at(path);
+                        if (c.net)
+                            sheet.warnings.emplace_back(pin_pos,
+                                                        "Pin connected to net: " + block->get_net_name(c.net->uuid));
+                    }
+                }
 
                 for (auto &it_line : sheet.net_lines) {
                     LineNet &line = it_line.second;
@@ -782,6 +792,21 @@ void Schematic::expand_connectivity(bool careful)
                 }
             }
         }
+        for (const auto &[uu_sym, sym] : sheet.block_symbols) {
+            for (const auto &[uu_port, port] : sym.symbol.ports) {
+                if (port.net) {
+                    auto pin_pos = sym.placement.transform(port.position);
+                    if (port.connection_count == 0) {
+                        if (sym.block_instance->connections.count(port.net)) {
+                            auto &c = sym.block_instance->connections.at(port.net);
+                            if (c.net)
+                                sheet.warnings.emplace_back(pin_pos, "Port connected to net: "
+                                                                             + block->get_net_name(c.net->uuid));
+                        }
+                    }
+                }
+            }
+        }
     }
 
     // warn bus rippers
@@ -1324,6 +1349,10 @@ void Schematic::update_refs()
         for (auto &it_sym : sheet.bus_rippers) {
             it_sym.second.update_refs(sheet, *block);
         }
+        for (auto &[uu, tie] : sheet.net_ties) {
+            tie.net_tie.update(block->net_ties);
+            tie.update_refs(sheet);
+        }
     }
 }
 
