@@ -1,6 +1,6 @@
 major = 2
-minor = 2
-micro = 0
-name = "Halo"
+minor = 3
+micro = 1
+name = "Iceblink"
 
 string = ".".join(str(x) for x in (major, minor, micro))
